# Hint for Emacs: -*- perl -*-
#
# itchconfig/itchcop.pm - helper library for auto-generated configure.pl files
# This file is part of itchconfig (<http://itchconfig.sourceforge.net/>), a
# config tool created by Arne Thomassen; itchconfig is basically released under
# certain versions of the GNU General Public License and WITHOUT ANY WARRANTY.
# Read the file COPYING for license details, README for program information.
# Copyright (C) 2002 Arne Thomassen <arne@arne-thomassen.de>

# "cop" as in "configure.pl" :-)

use Fcntl;
use itchlib;

my ($package_name, $package_name_conv, $package_version,$package_version_major,
    $package_version_minor, $package_version_micro, $package_version_extra,
    $package_makefile, $package_config_h, $substopt, $ensure_launchdir,
    $preserve_makefile, @mf_phony, @mf_mostlyclean, @mf_clean, @mf_distclean);

# snippets of the configure script
my @sh = ();
use constant SH_PART_HEADER  => 0;
use constant SH_PART_INIT    => 1;
use constant SH_PART_ARG     => 2;
use constant SH_PART_VERSION => 3;
use constant SH_PART_SLANG0  => 4;
use constant SH_PART_SLANG   => 5;
use constant SH_PART_CODE    => 6;

# command-line argument help texts for the configure script
my @sh_arg = ();
my $need_sh_arg_disable = 0;
my $need_sh_arg_assign = 0;
my $need_sh_arg_dvhelp = 0;
my @sh_argtitle =
  ('General Options', 'Installation Options', 'Installation Paths',
   'Package Options');
use constant SH_ARGPART_GENERAL     => 0;
use constant SH_ARGPART_INSTALL     => 1;
use constant SH_ARGPART_INSTALLPATH => 2;
use constant SH_ARGPART_PACKAGE     => 3;
use constant SH_ARGPART_MAX         => 3;
my @helpprepare = ();
my @helpprepare_installpath = ();

my @sh_require_abspath = (); # what to check after arg handling in "configure"
my @sh_require_abspath2 = (); # what to check before writing configure.act

my %is_slang_known = ('c' => 1);
my %is_slang_needed = ();
my %check_c_headers = ();

# distribution-related stuff
my $sh_dist_path = ''; my $sh_dist_collector = ''; my $sh_dist_compressor = '';

my @act_arg = (); # install-path command-line argument stuff for configure.act

my $tempdir = '';
my $ic_int_config_h = 'int_config.h'; # configure's _internal_ config.h file
my $config_guess = 'config.guess';
my $sourcefile = 'configure.base';
my $destfile = 'configure';
my $is_subconf = 0; # is sub-configuration?
my $using_sub_add = 0;
my @subconf_dist = ();
my $platforms = 1;
  # Unix-like platforms only (currently); IMPLEMENTME: IC_META_PLATFORM

my $might_use_infiles = 0;
my $might_install_something = 0;
my $might_install_programs = 0;
my $emitted_install_prefix = 0;
my $emitted_install_archprefix = 0;
my $emitted_egrep = 0;
my $might_make_binaries = 0;
my $might_create_extra_targets = 0;
my $might_change_default_target = 0;
my $need_convpath = 0;
my $need_dbgfd = 0;
my $need_c_definc = 0;
my $need_c_cpp = 0;
my $emitted_c_stdc = 0;
my $max_sed_lines = 48; # maximum number of lines in a sed script file

# File descriptors used by "configure"; we intentionally don't ever use 3 and 4
# because they are reserved for special stuff on some arcane systems.
# FIXME: don't use descriptors beyond 9 - some stupid shells can't handle them!
my ($fd_sh_null, $fd_sh_temp_makefile, $fd_sh_temp_config_h, $fd_sh_temp_subst,
    $fd_sh_log, $fd_sh_act, $fd_sh_temp_temp)
 = (5, 6, 7, 8, 9, 6, 11);

my ($infile_suffix, $infile_suffix_re, $infile_suffix_re_sh);
&set_infile_suffix('.in');

my $sh_exit_success = '{ :; exit 0; }';
my $sh_exit_failure = '{ (exit 1); exit 1; }';

my $headertextbase =
'# This configuration script file has been auto-generated by
# itchconfig 0.0.4 <http://itchconfig.sourceforge.net/>.
# Copyright (C) 2002 Arne Thomassen <arne@arne-thomassen.de>
# This configuration script file is free software WITHOUT ANY WARRANTY;
# you are given unlimited permission to copy, distribute and modify this file,
# as long as this notice is kept intact and you indicate file modifications.
';

my $caching = 0;
my $current_macro = '';
my $debug = 0; # values as in itchconfig script
my $found_ic_begin = 0; # for detecting duplicate IC_BEGIN in configure.base
my $found_ic_end = 0; # for detecting duplicate IC_END in configure.base

&itchlib_init('configure.pl');
&parse_arguments();

my $sop = ( ($platforms & ~1) ? (1) : (0) ); # "support other platforms?"
my $sh_slash = ($sop ? '\\\\/' : '/');
my $sh_slashsel = '[' . $sh_slash . ']'; # for regular expressions

my $sh_re_abspath = '[' . $sh_slash . '$]*';
if ($sop) { $sh_re_abspath .= ' | ?:' . $sh_slashsel . '*'; }

my $sh_progext = ($sop ? '$ic_progext' : '');
my $sh_me = 'ic_me=`echo "$0" | sed \'s,.*' . $sh_slashsel . ',,\'`';

# standard pre-/suffices for external identifiers
my %idfoofix = (
  # prefices:
  'version' => 'VERSION',
  'option-enable' => 'OPTION_', 'option-set' => 'OPTION_',
  'path-program' => 'PATH_PROGRAM_', 'path-install' => 'PATH_INSTALL_',
  'have-c-function' => 'HAVE_', 'have-c-library' => 'HAVE_',
  'have-c-header' => 'HAVE_', 'have-c-type' => 'HAVE_',
  # suffices:
  'exitcode-id' => '_E', 'output-id' => '_O'
);

my %installer_data =
(              # arch-dep., partial path, help text, flags, magic code
  'bin'     => [1, '', 'normal programs', 2],
  'include' => [0, '', 'C header files'],
  'info'    => [0, '', 'info documentation'],
  'lib'     => [1, '', 'object-code libraries', 4],
  'locale'  => [0, 'share', 'natural language texts', '',
'mcf=`echo "$%F%" | sed \'s,\.mo$,,\'`
%P%="$%P%/$mcf/LC_MESSAGES"
%F%="%PN%.mo"'],
  'man'     => [0, '', 'manual documentation', '',
'num=`echo "$%F%" | sed \'s,.*\.\([0123456789]\).*,\\1,\'`
%P%="$%P%/man$num"'],
  'sbin'    => [1, '', 'super-user programs', 2]
);
my @installer_magic = ();
my @installer_subst = ();
# IMPLEMENTME: configure should auto-detect the path for 'man' from the
# environment variable MANPATH?

1;

sub parse_arguments
{ foreach my $one (@ARGV)
  { if ($one =~ m/^\-\-sourcefile\=(.+)$/) { $sourcefile = $1; }
    elsif ($one =~ m/^\-\-output\=(.+)$/) { $destfile = $1; }
    elsif ($one =~ m/^\-\-tempdir\=(.+)$/) { $tempdir = $1; }
    elsif ($one =~ m/^\-\-debug\=([0-9]+)$/)
    { $debug = $1 + 0;
      if ($debug != 0) { $verbosity = 4; }
    }
    elsif ($one =~ m/^\-\-verbosity\=([0-4])$/) { $verbosity = $1 + 0; }
    else { &fatal_error("unrecognized option: $one"); }
  }
  if ($tempdir eq '') { &fatal_error('no temporary directory given'); }
}


# Section: General

sub convname
{ my ($name, $flags) = @_;
  if ($flags & 1) { $name =~ s/[^A-Za-z0-9]/\_/g; }
  else
  { $name =~ tr/a-z/A-Z/;
    $name =~ s/[^A-Z0-9]/\_/g;
  }
  return($name);
}

sub deny_subconf
{ if ($is_subconf) { &fatal_error("$_[0] not allowed in sub-configuration"); }
}

sub deny_toplevelconf
{ if ($found_ic_begin)
  { &fatal_error("$_[0] not allowed in top-level configuration"); }
}

sub cop_begin
{ my $behavior;
  &deny_subconf('IC_BEGIN');
  ($package_name, $package_version, $behavior, $package_makefile,
   $package_config_h) = @_;

  if ($found_ic_begin) { &fatal_error('found duplicate IC_BEGIN'); }
  $found_ic_begin = 1;

  if ($package_name eq '') { &fatal_error('IC_BEGIN: missing package name');}
  if ($package_version eq '') { &fatal_error('IC_BEGIN: missing version'); }
  $package_name_conv = &convname($package_name);

  if ($package_makefile eq '_') { $package_makefile = 'Makefile'; }
  if ($package_config_h eq '_') { $package_config_h = 'config.h'; }

  if ($behavior eq '') { $behavior = 0; }
  else { $behavior += 0; }

  my $prefix = (($behavior & 16) ? ($package_name_conv . '_') : '') .
    $idfoofix{'version'};
  $behavior &= ~16;
  &cop_substitute_behavior($behavior);

  &cop_substitute_begin();
  if ( ($package_version =~ tr/\ //) >= 2 ) # looks like a list
  { ($package_version_major, $package_version_minor, $package_version_micro,
     $package_version_extra) = split(/\s+/, $package_version, 4);
    $package_version = $package_version_major . '.' . $package_version_minor .
      '.' . $package_version_micro . $package_version_extra;
    &sh_substitute($prefix . '_MAJOR', $package_version_major);
    &sh_substitute($prefix . '_MINOR', $package_version_minor);
    &sh_substitute($prefix . '_MICRO', $package_version_micro);
    if ($package_version_extra ne '')
    { &sh_substitute($prefix . '_EXTRA', $package_version_extra); }
  }
  &sh_substitute($prefix, $package_version);
  &cop_substitute_end();
  $behavior &= ~1;
  &cop_substitute_behavior($behavior);
}

sub sh_open_log
{ my $do_append = ($_[0] & 1);
  return("\nexec $fd_sh_log" . ($do_append ? '>' : '') . ">configure.log\n");
}

sub sh_close_fd
{ my ($fd, $flags, $prettyprint) = @_;
  my $retval = $prettyprint . "exec $fd>&$fd_sh_null";
  if (!($flags & 1)) { $retval .= "\n"; }
  return($retval);
}

sub sh_abspath
{ my @path = @_;
  if ($#path < 0) { return; } # nothing to do
  my $paths = &sh_beautify(join(' ', @path), 14, ' \\');
  print OUT "\n# Make sure that paths are absolute\n",
      'for i_path in ', $paths,
'
do
eval ic_val=$`echo $i_path`
', &sh_debug('abspath: $ic_val - $i_path', 2), 'case $ic_val in
  ', $sh_re_abspath, ')
  ;;
  *)
', &sh_fatal_error('$i_path must be absolute, but is \"$ic_val\"', '  '), '
  ;;
esac
done

';
}

sub sh_sitecall
{ my $stage = $_[0];
  return('
if test -n "$ic_site_file" && test "x$ic_site_next" = x' . $stage . '; then
  ic_site_stage=' . $stage . '
  ic_site_next=
  . "$ic_site_file"
fi
');
}

sub insert_emacs_mark
# inserts a mark of the form "-*-foo-*-" to the beginning of a text (which
# will become the beginning of a file) to set an Emacs mode
{ my ($text, $mark) = @_;
  if ($text =~ s/^([^\n]*)//)
  { my $first_line = $1;
    my $len = 79 - length($mark) - length($first_line);
    if ($len > 0) { $first_line .= (' ' x $len) . $mark; }
    $text = $first_line . $text;
  }
  return($text);
}

sub escape_here
# escapes a text which will be used as a shell here-document
{ my $text = $_[0];
  $text =~ s/\\/\\\\/g; $text =~ s/\$/\\\$/g; $text =~ s/\`/\\\`/g;
  return($text);
}

sub escape_here_twice
# like escape_here(), just double indirection
{ my $text = $_[0];
  $text =~ s/\\/\\\\\\\\/g; $text =~ s/\$/\\\\\\\$/g; $text =~ s/\`/\\\\\\\`/g;
  return($text);
}

sub write_configure_script
# This is the worst sub-routine in the whole project. You have been warned! :-)
{ if (!sysopen(OUT, $destfile, O_CREAT | O_TRUNC | O_WRONLY, 0544))
  { &fatal_error('cannot write "configure"'); }

  my $sh_bournecompat = 'if test -n "${ZSH_VERSION+set}" && (emulate sh) >&' . $fd_sh_null . ' 2>&1; then
  emulate sh
  NULLCMD=:
elif test -n "${BASH_VERSION+set}" && (set -o posix) >&' . $fd_sh_null . ' 2>&1; then
  set -o posix
fi';

  my $sh_IFS = 'ic_nl=\'
\'
IFS=" 	$ic_nl"
export IFS';

  my $sh_unsetcompat = 'if (FOO=FOO; unset FOO) >&' . $fd_sh_null . ' 2>&1; then
  ic_unset=unset
else
  ic_unset=false
fi';

  # work-around for UWIN ksh before 3.0:
  my $sh_eps = '$ic_unset ENV MAIL MAILPATH
PS1=\'$ \'
PS2=\'> \'
PS4=\'+ \'';

  # "langry" = "lang" + "angry"...
  my $langry = 'for i_x in LANG LANGUAGE LC_ALL LC_COLLATE LC_CTYPE LC_MESSAGES LC_NUMERIC LC_TIME
do
  if (set +x; test -n "`(eval $i_x=C; export $i_x) 2>&1`"); then
    eval $i_x=C; export $i_x
  else
    $ic_unset $i_x
  fi
done';

  my $allcompat = "$sh_bournecompat\n\n$sh_IFS\n\n$sh_unsetcompat\n\n$sh_eps\n\n$langry\n";

  my $x = '# configure - ' . ($is_subconf ? 'sub-' : '') .
    "configuration script for $package_name $package_version\n#\n" .
    $headertextbase;
  my $headertext_v = $x . $sh[SH_PART_HEADER];
  my $headertext_v2 = $x . $sh[SH_PART_VERSION];
  $headertext_v2 =~ s/^\#\ configure/\#\ \$ic\_me/;
  $headertext_v2 =~ s/\#(\ )?//g;
  $headertext_v2 =~ s/\n\n/\n/g;

  # Code for beginning (with tons of compatibility rubbish...)

  print OUT "#! /bin/sh\n$headertext_v
exec $fd_sh_null>/dev/null\n$allcompat\n$sh_me
ic_version='$progversion'
ic_package_name='$package_name'
ic_package_version='$package_version'", '
ic_fatal_error=echo
ic_show_message=echo
ic_show_info=false
';

  if ($is_subconf)
  { print OUT 'case \$1 in
--int-cfg | --int-dist)
ic_sub_fullpath="$2"
;;
*)
ic_sub_fullpath=
;;
esac
';
  }
  elsif ($using_sub_add) { print OUT "ic_sub_fullpath=.\n"; }

  print OUT '
ic_site_file=
ic_site_next=
i_site="${ITCHCONFIG_SITE-}"
if test -n "$i_site"; then
  for i_x in \'-', $package_name, '\' \'\'
  do
    i_s=`echo "$i_site" | sed "s,%P%,$i_x,g"`
    if test -n "$i_s" && test -f "$i_s"; then
      ic_site_file="$i_s"
      ic_site_next=init1
      break
    fi
  done
fi
', &sh_sitecall('init1'), '
ic_AZ=ABCDEFGHIJKLMNOPQRSTUVWXYZ
ic_az=abcdefghijklmnopqrstuvwxyz
ic_Az=$ic_AZ$ic_az
ic_Az09=${ic_Az}0123456789

# Override env vars?
for i_arg
do
  case $i_arg in
  [_$ic_Az]*=*)
  i_var=`expr "x$i_arg" : \'x\([^=]*\)=\'`
  i_val=`expr "x$i_arg" : \'x[^=]*=\(.*\)\' | sed "s/\'/\'\\\\\\\\\\\\\\\\\'\'/g"`
  eval "$i_var=\'$i_val\'"; export $i_var
  ;;
  esac
done

SHELL="${ITCHCONFIG_SHELL-/bin/sh}"
export SHELL
ic_makedir="${MKDIRPROG-mkdir}"
if test "${PATH_SEPARATOR+set}" != set; then
';

  if (!$sop) { print OUT '  PATH_SEPARATOR=:'; }
  else # need an extra test
  { print OUT
'  cat >configure.test <<EOF
#! $SHELL
', $sh_exit_success, '
EOF
  chmod u+rx configure.test
  if (PATH="/nonexistent;."; configure.test) >&', $fd_sh_null, ' 2>&1; then
    PATH_SEPARATOR=\';\'
  else
    PATH_SEPARATOR=:
  fi
  rm -f configure.test >&', $fd_sh_null, ' 2>&1';
  }

print OUT '
fi
$ic_unset CDPATH
';
# || test "${CDPATH+set}" != set || { CDPATH=$PATH_SEPARATOR; export CDPATH; }

  if ( ($is_subconf) and ($sh_dist_collector ne '') )
  { print OUT "case \$1 in
--int-dist)
;;
esac
";
  }

  if ($sop) { print OUT "ic_progext=\${PROGEXT-}\n"; }

  print OUT '
case `echo "testing\c"; echo 1,2,3`,`echo -n testing; echo 1,2,3` in
  *c*,-n*) ic_ECHO_N= ic_ECHO_C=\'
\' ic_ECHO_T=\'	\' ;;
  *c*,*  ) ic_ECHO_N=-n ic_ECHO_C= ic_ECHO_T= ;;
  *)      ic_ECHO_N= ic_ECHO_C=\'\c\' ic_ECHO_T= ;;
esac

';

  if ($debug & 8) { print OUT "ic_log=false\n"; }
  if ($debug & 16) { print OUT "ic_do_debug=false\nic_debug=false\n"; }
  if ($need_dbgfd) { print OUT "ic_dbgfd=$fd_sh_null\n"; }
  if ($config_guess !~ m/^[\.\/\~]/) { $config_guess = './' . $config_guess; }

  print OUT $sh[SH_PART_INIT], '
', &sh_sitecall('init2'), '
# Parse args
i_premature_exit=a
for i_arg
do
';

  if ($need_sh_arg_assign)
  { print OUT "\n", 'i_aval=`expr "x$i_arg" : \'x[^=]*=\(.*\)\'`', "\n";
  }

print OUT '
case $i_arg in
', $sh[SH_PART_ARG], '
  --site* | [_$ic_Az]*=*)
  ;;

  *)
', &sh_fatal_error('unrecognized option: $i_arg. Try \\"./$ic_me --help\\"!', '  '), '
  ;;

esac
done

case $i_premature_exit in', "
a)
  ;;
b)
  cat <<EOF
$headertext_v2", "EOF
  $sh_exit_success
  ;;
c)
  echo \"\$ic_me report for $package_name $package_version (by itchconfig $progversion):\"
  if test -f \"$config_guess\"; then
    i_xv=\"`$config_guess --version 2>&$fd_sh_null | sed 1q`\"
    i_x=\"`$config_guess 2>&$fd_sh_null`\"
    echo \"config.guess (\$i_xv) says: \$i_x\"
  fi
  i_x=\"`uname -a 2>&$fd_sh_null`\"
  echo \"uname -a says: \$i_x\"
";

  if ($package_makefile ne '')
  { print OUT "  i_x=\"`make --version 2>&$fd_sh_null | sed 1q`\"
  echo \"make version: \$i_x\"
";
  }

  if ($is_slang_needed{'c'})
  { print OUT "  i_x=\"`\$ic_CC --version 2>&$fd_sh_null | sed 1q`\"
  echo \"C compiler: \$ic_CC, \$i_x\"
";
  }

  print OUT "  $sh_exit_success
  ;;
d)
";

  @helpprepare = (); @helpprepare_installpath = (); # FIXME/IMPLEMENTME?

  if ($#helpprepare >= 0)
  { foreach my $one (@helpprepare) { print OUT "  $one\n"; }
  }

  if ($#helpprepare_installpath >= 0)
  { # setup the help text variables for the default installation paths
    print OUT '  for i_p in ', join(' ', @helpprepare_installpath), '
  do
    eval i_x=$`echo "$i_p"`
    eval ${i_p}_h=`echo "$i_x" | sed \'s,${ic_path_prefix},PREFIX,g;s,${ic_path_archprefix},ARCHPREFIX,g\'`
#    echo "p=$i_p, x=$i_x"
  done
';
  }

  print OUT "  cat <<EOF
\\`\$ic_me' configures $package_name $package_version to adapt to your computer.
Script generated by itchconfig $progversion <http://itchconfig.sourceforge.net/>.
Usage: ./\$ic_me [option] ...\n";

  if ($need_sh_arg_dvhelp)
  { print OUT "Default values are shown in brackets below.\n";
  }

  my $code = '';
  if ($need_sh_arg_disable)
  { unshift(@{$sh_arg[SH_ARGPART_PACKAGE]},
      [ '--disable-FEATURE', 'the opposite of --enable-FEATURE' ]);
  }

  for (my $i = 0; $i <= SH_ARGPART_MAX; $i++)
  { my @x = @{$sh_arg[$i]};
    if ($#x >= 0) # need to output something
    { $code .= "\n";
      my $title = $sh_argtitle[$i];
      if ($title ne '') { $code .= "$title:\n"; }
      my $maxnamelen = 0;
      foreach my $one (@x)
      { my $name = @{$one}[0];
        my $len = length($name);
        if ($maxnamelen < $len) { $maxnamelen = $len; }
      }
      $maxnamelen += 2;
      foreach my $one (@x)
      { my ($name, $help) = @{$one};
        if ($help ne '')
        { my $diff = $maxnamelen - length($name);
          if ($diff > 0) { $name .= ' ' x $diff; }
        }
        $code .= $name . &sh_beautify($help, $maxnamelen) . "\n";
      }
    }
  }
  print OUT $code, "EOF\n  $sh_exit_success\n  ;;\n";

  if ( (!$is_subconf) and ($sh_dist_collector ne '') )
  { print OUT "e)\n$sh_dist_collector\n", &sh_dist_subcall('.'),
      "$sh_dist_compressor\n  $sh_exit_success\n  ;;\n";
  }

  print OUT "esac\n", $ensure_launchdir;

  if ($debug & 8) { print OUT &sh_open_log(0); }
  if ($debug & 16)
  { print OUT "if \$ic_do_debug; then\n  ic_debug=echo\n  ic_log=echo\n";
    if ($need_dbgfd) { print OUT "  ic_dbgfd=$fd_sh_log\n"; }
    print OUT "fi\n";
  }
  print OUT &sh_log('Running configure', 2);

  &sh_abspath(@sh_require_abspath);

  print OUT "\n# Create a temporary directory\n";

  if ($debug & 16) { print OUT "\$ic_do_debug ||\n"; }
  print OUT '{ trap \'exit_status=$?; rm -rf $ic_tempdir && exit $exit_status\' 0
  trap \'', $sh_exit_failure, '\' 1 2 13 15
}
: ${TMPDIR=/tmp}
{ ic_tempdir=`(umask 077 && mktemp -d -q "$TMPDIR/icXXXXXX") 2>&', $fd_sh_null, '` &&
  test -n "$ic_tempdir" && test -d "$ic_tempdir"
} ||
{ ic_tempdir=$TMPDIR/ic$$-$RANDOM
  (umask 077 && $ic_makedir $ic_tempdir)
} ||
{
', &sh_fatal_error('cannot create a temporary directory in $TMPDIR', '  '),
'
}
', &sh_debug('Created the temporary directory $ic_tempdir', 1), '
$ic_show_info "Configuring ' . $package_name . ' ' . $package_version . '..."
ic_testfile=$ic_tempdir/test
ic_substfile=$ic_tempdir/subst
exec ', $fd_sh_temp_subst, '>$ic_substfile
';

  if ($is_slang_needed{'c'})
  { # need our own, internal config.h file for several tests
    print OUT 'ic_int_config_h=$ic_tempdir/' . $ic_int_config_h . '
echo >$ic_int_config_h
';
  }

  if ($need_c_cpp)
  { &cop_message_checking('how to run the C pre-processor', SH_PART_SLANG0);
    # We must make sure that the stuff is really only run through the
    # pre-processor, not through the compiler; that's why there's the text
    # "Syntax error" in the test code. Also we must check whether $CPP means a
    # directory (might e.g. happen on Suns).
    &emit_sh(SH_PART_SLANG0,
      'if test -n "$CPP" && test -d "$CPP"; then', '  CPP=', 'fi',
      'if test -n "$CPP"; then', '  ic_CPP="$CPP"', 'else', '  ic_CPP=');
    &sh_c_write_testfile('#include <assert.h>
  Syntax error', SH_PART_SLANG0);
    &emit_sh(SH_PART_SLANG0,
      '  for CPP in "$ic_CC -E" "$ic_CC -E -traditional-cpp" /lib/cpp',
      '  do', '    if $CPP $ic_CPPFLAGS0 $ic_testfile.c >&' . $fd_sh_null . ' 2>&1; then',
      '      ic_CPP="$CPP"', '      break', '    fi', '  done', 'fi');
    # FIXME: we need further tests here!
    &cop_message_checkresult('$ic_CPP', SH_PART_SLANG0);
    &emit_sh(SH_PART_SLANG0, 'if test -z "$ic_CPP"; then',
      &sh_fatal_error('cannot find out how to run the C pre-processor; set the environment variable CPP and try again', '  '), 'fi');
  }

  if ($need_c_definc)
  { print OUT 'ic_c_definc=\'#include <stdio.h>
#if HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif
#if HAVE_SYS_STAT_H
# include <sys/stat.h>
#endif
#if STDC_HEADERS
# include <stdlib.h>
# include <stddef.h>
#else
# if HAVE_STDLIB_H
#  include <stdlib.h>
# endif
#endif
#if HAVE_STRING_H
# if !STDC_HEADERS && HAVE_MEMORY_H
#  include <memory.h>
# endif
# include <string.h>
#endif
#if HAVE_STRINGS_H
# include <strings.h>
#endif
#if HAVE_INTTYPES_H
# include <inttypes.h>
#else
# if HAVE_STDINT_H
#  include <stdint.h>
# endif
#endif
#if HAVE_UNISTD_H
# include <unistd.h>
#endif
\'

';
    &cop_c_headers_stdc('', '', 1);
    &c_check_headers('inttypes.h', 'memory.h', 'stdint.h', 'stdlib.h',
      'string.h', 'strings.h', 'unistd.h', 'sys/types.h', 'sys/stat.h');
  }

  my @cch = keys %check_c_headers;
  if ($#cch >= 0) { &cop_c_headers(join(' ', sort @cch), '', 1 | 2); }

  print OUT $sh[SH_PART_SLANG0], $sh[SH_PART_SLANG];
  if ($need_convpath)
  { print OUT '
ic_convpath=`echo "$PATH" | sed "s,$PATH_SEPARATOR, ,g"`
', &sh_debug('ic_convpath: $ic_convpath', 2);
  }

  if ($caching >= 2) # read any cached values
  { print OUT '
if test -f "configure.cache"; then
  . configure.cache
fi
';
  }

  if ($package_makefile ne '')
  { my $headertext_m = $headertextbase;
    $headertext_m =~ s/configuration\ script\ file/Makefile\ for\ $package_name\ $package_version/;
    $headertext_m =~ s/configuration\ script\ file/Makefile/g;
    $headertext_m = &insert_emacs_mark($headertext_m, '-*-Makefile-*-');
    $headertext_m .= "\nall: default-target\n";
    print OUT "\nic_makefile=\$ic_tempdir/Makefile\nexec ",
      $fd_sh_temp_makefile, '>$ic_makefile', "\n",
      'cat >&', $fd_sh_temp_makefile, " <<EOF\n", $headertext_m,
      'SHELL=$SHELL';
#   if ($is_slang_needed{'c'}) { print OUT "\nCC=\$ic_CC"; }
    print OUT "\n\nEOF\n";
    if ($might_create_extra_targets) { print OUT "ic_extra_targets=\n"; }
    if ($might_change_default_target)
    { print OUT "ic_default_target=all-binaries\n"; }
  }

  if ($package_config_h ne '')
  { my $headertext_h = $headertextbase;
    $headertext_h =~ s/configuration\ script\ file/config\.h\ file\ for\ $package_name\ $package_version/;
    $headertext_h =~ s/configuration\ script\ file/config\.h\ file/g;
    $headertext_h =~ s/^\#/\/\*/;
    $headertext_h =~ s/\#\ /\ \ \ /g;
    $headertext_h .= '*/';
    $headertext_h = &insert_emacs_mark($headertext_h, '-*-C-*-');

    print OUT "\nic_config_h=\$ic_tempdir/config.h\nexec ",
      $fd_sh_temp_config_h, '>$ic_config_h', "\n",
      'cat >&', $fd_sh_temp_config_h, " <<EOF\n", $headertext_h, "\n\nEOF\n";
  }

  # The "real" code

  print OUT "\n", $sh[SH_PART_CODE];
  &sh_abspath(@sh_require_abspath2);

  # Code for completing the temporary Makefile

  if ($package_makefile eq '') { goto no_makefile; }
  my $ntd = "\t\@echo \"Nothing to do!\" >&2";
  push(@mf_phony, 'all', 'all-binaries', 'default-target');

  if ($might_make_binaries)
  { print OUT '
if test -n "$ic_binaries"; then
  allbin="all-binaries:"
  for bin in $ic_binaries
  do
    allbin="$allbin binary-$bin"
  done
  cat >&', $fd_sh_temp_makefile, ' <<EOF
$allbin

EOF
  for bin in $ic_binaries
  do
    eval src=$`echo "ic_binary_${bin}_c_sources"`
    eval cflags=$`echo "ic_binary_${bin}_cflags"`
    eval libs0=$`echo "ic_binary_${bin}_libs"`
    libs=
    for lib in $libs0
    do
      case $lib in
        -*) ;;
        *) lib="-l$lib" ;;
      esac
      libs="$libs $lib"
    done
    dep="$src', ( ($package_config_h ne '') ? ' ' . $package_config_h : '' ),'"
    ovals=
    if test -n "$src"; then
      for src1 in $src
      do
        case $src1 in
          *.c)
          oval=`expr "x$src1" : \'x*\\(.*\\)\.c\$\'`
          oval=$oval.$ic_objext
          ovals="$ovals $oval"
          cat >&', $fd_sh_temp_makefile, ' <<EOF
$oval: $dep
', "\t", '$ic_CC $cflags $ic_CPPFLAGS0 -c $src1 -o $oval
EOF
          ;;
        esac
      done
    fi
    cat >&', $fd_sh_temp_makefile, ' <<EOF
binary-$bin: $ovals
', "\t", '$ic_CC $cflags $ic_CPPFLAGS0 $ic_LDFLAGS $ovals $LIBS $libs -o $bin', $sh_progext, '

EOF
  done
else
  cat >&', $fd_sh_temp_makefile, " <<EOF
all-binaries:
$ntd
EOF
fi
";
  # FIXME: don't simply say "dep=$src" above for C source dependencies!
  }
  else
  { print OUT 'cat >&', $fd_sh_temp_makefile, " <<EOF
all-binaries:
$ntd
EOF
";
  }

  if ($might_install_something)
  { push(@mf_phony, 'install');
    print OUT '
if test -n "$ic_installs"; then
  allinst="install:"
  for inst in $ic_installs
  do
    eval ic_val=$`echo "ic_install_${inst}_files"`
    if test -n "$ic_val"; then
      eval path=$`echo "ic_path_${inst}" | sed "s,[^$ic_Az09],_,g"`
      eval pfinal0="$path"
      eval pfinal="$pfinal0"
      eval perm=$`echo "ic_install_${inst}_perm"`
      eval magic=$`echo "ic_install_${inst}_magic"`
      allinst="$allinst install-$inst"
      cat >&', $fd_sh_temp_makefile, ' <<EOF
install-$inst:
', "\t", '@./configure.act --int-install $perm $magic $pfinal "$ic_val"
EOF
    fi
  done
  echo "$allinst" >&', $fd_sh_temp_makefile, '
fi

';
  }

  if ($might_create_extra_targets)
  { print OUT 'if test -n "$ic_extra_targets"; then
  for target in $ic_extra_targets
  do
    eval dep=$`echo "ic_extra_target_${target}_dep"`
    cat >&', $fd_sh_temp_makefile, ' <<EOF

$target: $dep
EOF
    cat >&', $fd_sh_temp_makefile, ' <$ic_tempdir/target_$target
  done
fi
';
  }

  push(@mf_phony, 'clean', 'mostlyclean', 'distclean', 'maintainer-clean');
  my $mf_clean = join(' ', sort @mf_clean);
  if ($mf_clean ne '') { $mf_clean = "\t-\@rm -f $mf_clean"; }
  my $pch = ( ($package_config_h ne '') ? (' ' . $package_config_h) : ('') );

  my $mf_maint = 'maintainer-clean: distclean';
  if (!$is_subconf)
  { $mf_maint .= "\n\t-\@rm -rf";
    if ($sh_dist_path ne '') { $mf_maint .= ' ' . $sh_dist_path; }
    $mf_maint .= ' *~'; # FIXME!
  }

  if ($#mf_mostlyclean < 0) { print OUT "mf_mostlyclean=\n"; }
  else
  { print OUT "mf_mostlyclean=\"\t-\@rm -f\"", '
for mfm in ', join(' ', sort @mf_mostlyclean), '
do
  mf_mostlyclean="$mf_mostlyclean $mfm$ic_objext"
done
';
  }

  print OUT 'cat >&', $fd_sh_temp_makefile, ' <<EOF

mostlyclean:
$mf_mostlyclean
clean: mostlyclean
', $mf_clean, '
distclean: clean
', "\t-\@rm -f configure.act$pch\n";

  if ($preserve_makefile)
  { print OUT "\t-\@if test -f \"Makefile-classical\"; then mv -f Makefile-classical Makefile; else :; fi\n";
  }
  else { print OUT "\t-\@rm -f $package_makefile\n"; }

  if ($#mf_distclean >= 0)
  { print OUT "\t-\@rm -f ", join(' ', sort @mf_distclean), "\n";
    # IMPLEMENTME: &sh_beautify(join(....), 16, ' \\\\', 1), "\n";
  }

  print OUT "$mf_maint\n";

  # Disable any make magic, don't rely on buggy implicit rules'n'stuff...
  print OUT '
default-target: ',
($might_change_default_target ? '$ic_default_target' : 'all-binaries'), '
.PHONY: ', &sh_beautify(join(' ', sort @mf_phony), 8, ' \\\\\\\\'), '
.SUFFIXES:
.DELETE_ON_ERROR:
.NOEXPORT:
EOF
';

  no_makefile:

  # Code for file-handle closing

  if ($package_makefile ne '') { print OUT &sh_close_fd($fd_sh_temp_makefile);}
  if ($package_config_h ne '') { print OUT &sh_close_fd($fd_sh_temp_config_h);}
  print OUT &sh_close_fd($fd_sh_temp_subst);

  # Code for writing configure.act

  my $header_a = '# configure.act - configured actions for ' .
    "$package_name $package_version\n#\n$headertextbase";
  my $headertext_av = $header_a;
  $headertext_av =~ s/^\#\ configure\.act/\#\ \$ic\_me/;
  $headertext_av =~ s/\#(\ )?//g;
  $headertext_av =~ s/\n\n/\n/g;

  my $bifs = "$allcompat\n$sh_me\nic_fatal_error=%ic_fatal_error\nic_show_info=%ic_show_info\n$ensure_launchdir";

  if ($might_install_something)
  { # CHECKME: run chown on installed files?
    $bifs .= '
case $1 in
  --int-install)
  mode="$2"
  magic="$3"
  initial_path="$4"
  files="$5"
  act="${ITCHCONFIG_ACT-}"
  ic_makedir="${MKDIRPROG-mkdir}"
';

    if ($might_install_programs)
    { $bifs .= '  ic_program_prefix="%ic_program_prefix"
  ic_program_suffix="%ic_program_suffix"
';
    }

    $bifs .= '  checked_path=
  for one_file in $files
  do
    path="$initial_path"
    file=`echo "$one_file" | sed \'s,.*' . $sh_slashsel . ',,\'`
';

    if ($#installer_magic >= 0) # need magic path/filename conversions
    { $bifs .= "    case \$magic in\n    m0) ;;\n"; # most likely case m0 first
      my $count = 0;
      foreach my $magic (@installer_magic)
      { $count++;
        $bifs .= "    m$count)\n" . $magic . "\n    ;;\n";
      }
      $bifs .= "    *)\n" .
        &sh_fatal_error('unknown installer magic \\"$magic\\"', '    ') .
        "\n    ;;\n    esac\n";
    }

    # CHECKME: use "cp -p" instead of "cp"?
    $bifs .= '    if test "x$checked_path" != "x$path"; then
      checked_path="$path"
      parts=`echo "$path" | sed \'s,/, ,g\'`
      p=
      for part in $parts
      do
        p="$p/$part"
        if test -d "$p" || $act $ic_makedir "$p"; then :; else
' . &sh_fatal_error('cannot create directory \\"$p\\"', '          ') . '
        fi
      done
    fi
    dest_file="$path/$file"
    if test -f "$dest_file"; then
      temp_file=$path/#itch.$$#
      $act trap "mv -f $temp_file $dest_file" 1 2 13 15
      $act mv -f $dest_file $temp_file
      $act cp $one_file $dest_file
      $act rm -f $temp_file
    else
      $act cp $one_file $dest_file
    fi
    $act chmod $mode $dest_file
  done
  ' . $sh_exit_success . '
  ;;
esac
';
  }

#  my $need_act_arg_assign = ( ($#act_arg >= 0) ? 1 : 0 );
# IMPLEMENTME!
  my $need_act_arg_assign = 0;
  @act_arg = ();

  $bifs .= "\n\n# Parse args
i_premature_exit=a
for i_arg
do\n";

  if ($need_act_arg_assign)
  { $bifs .= "\n" . 'i_aval=`expr "x$i_arg" : \'x[^=]*=\(.*\)\'`' ."\n";
  }

  $bifs .= '
case $i_arg in

  --version | -V)
  i_premature_exit=b
  break
  ;;

  --help)
  i_premature_exit=c
  break
  ;;

  *)
' . &sh_fatal_error('unrecognized option: $i_arg. Try \\"./$ic_me --help\\"!', '  ') . '
  ;;

esac
done

case $i_premature_exit in' . "
a)
  ;;
b)
  cat <<EOF
$headertext_av" . "EOF
  $sh_exit_success
  ;;
c)
  cat <<EOF
\\`\$ic_me' performs configured actions for $package_name $package_version.
Script generated by itchconfig $progversion <http://itchconfig.sourceforge.net/>.
Usage: ./\$ic_me [option] ...
";

  if ($need_act_arg_assign)
  { $bifs .= "Default values are shown in brackets below.\n"; }

  $bifs .= '
General Options:
--version, -V  print the version information and exit
--help         print this help and exit
';

  if ($#act_arg >= 0)
  { $bifs .= "\nInstallation Paths:\n";
    foreach my $one (@act_arg)
    { my ($option, $help, $var) = @{$one};
      $bifs .= "--$option: $help [%$var]\n"; # FIXME!
    }
  }

  $bifs .= "EOF
  $sh_exit_success
  ;;
esac
";

  $bifs = &escape_here($bifs);
  $bifs =~ s/\%/\$/g; # nominated for the infamous Horrible Mess Award... :-)

  print OUT '
$ic_show_info "Writing configure.act"
exec ', $fd_sh_act, '>configure.act
cat >&', $fd_sh_act, ' <<EOFACT
#! $SHELL
', $header_a, "
exec $fd_sh_null>/dev/null\n$bifs\n";

  if ($debug & 8)
  { print OUT "ic_log=\$ic_log\n", &sh_open_log(1),
      &sh_log('Running configure.act', 2);
  }
  print OUT "EOFACT\n";

  if ($package_makefile ne '')
  { if ($preserve_makefile)
    { print OUT "\n", 'cat >&', $fd_sh_act, ' <<EOFACT
if test -f "Makefile-classical"; then :; else
  mv -f Makefile Makefile-classical
fi
EOFACT
';
    }
    my $wtm = 'Writing the Makefile';
    if ($package_makefile ne 'Makefile') { $wtm .= " ($package_makefile)"; }
    print OUT "\ncat >&$fd_sh_act <<EOFACT\n\\\$ic_show_info \"$wtm\"\nEOFACT\n",
      'echo "cat >', $package_makefile, ' <<EOFINACT" >&',
      $fd_sh_act, "\n", 'cat >&', $fd_sh_act, ' <$ic_makefile', "\n",
      'echo "EOFINACT" >&', $fd_sh_act, "\n";
  }

  if ($package_config_h ne '')
  { my $wtc = 'Writing the config.h file';
    if ($package_config_h ne 'config.h') { $wtc .= " ($package_config_h)"; }
    print OUT "\ncat >&$fd_sh_act <<EOFACT\n\\\$ic_show_info \"$wtc\"\nEOFACT\n",
      'echo "cat >', $package_config_h, ' <<EOFINACT" >&',
      $fd_sh_act, "\n", 'cat >&', $fd_sh_act, ' <$ic_config_h', "\n",
      'echo "EOFINACT" >&', $fd_sh_act, "\n";
  }

  if ($might_use_infiles)
  { print OUT '
if test -n "$ic_infiles"; then
  echo >&', $fd_sh_act, '
  echo "cat >configure.subst <<EOFINACT" >&', $fd_sh_act, '
  cat >&', $fd_sh_act, ' <$ic_substfile
  cat >&', $fd_sh_act, ' <<EOFACT
EOFINACT
further_infiles="$ic_infiles"
while test -n "\\$further_infiles"
do
  infiles="\\$further_infiles"
  further_infiles=
  for infile in \\$infiles
  do
    file=\\`echo "\\$infile" | sed \'s,', &escape_here($infile_suffix_re_sh), '\\$,,\'\\`
    \\$ic_show_info "Creating \\$file from \\$infile"
    sed -f configure.subst \\$infile >\\$file 2>&', $fd_sh_null, '
    if expr "\\$file" : \'', &escape_here($infile_suffix_re_sh), '\$\' >&', $fd_sh_null, ' 2>&1; then
      further_infiles="\\$further_infiles \\$file"
    fi
  done
done
rm -f configure.subst >&', $fd_sh_null, ' 2>&1
EOFACT
fi
';
    # FIXME: split configure.subst into small pieces (-> $ic_sed_max)!
  }

  print OUT "\n", 'echo "', $sh_exit_success, '" >&', $fd_sh_act, "\n";

  # Code for executing configure.act

  if ($debug & 8) { print OUT &sh_close_fd($fd_sh_log); }
  print OUT &sh_close_fd($fd_sh_act), '
$ic_show_info "Executing configure.act"
chmod u+rx configure.act
$SHELL configure.act
';

  # Code for finishing

  print OUT "\n$sh_exit_success\n";
  close(OUT);
}

sub sh_beautify
# formats a text nicely
{ my ($text, $prologlen, $suffix, $flags) = @_;
  my $retval = '';
  my $is_first_line = 1;
  my $maxlen = 80 - $prologlen - length($suffix);
  my $foolen = 0; # FIXME for @mf_distclean! ( ($flags & 1) ? (8) : (0) );
  while ( (length($text) + $foolen >= $maxlen) and # text is too long
          ( (my $pos = rindex($text, ' ', $maxlen - $foolen)) > 0 ) )
  { my $t = substr($text, 0, $pos);
    $text = substr($text, $pos + 1);
    if ($is_first_line) { $is_first_line = 0; }
    else { $t = (' ' x $prologlen) . $t; }
    $retval .= $t . $suffix. "\n";
    $foolen = 0;
  }
  if (!$is_first_line) { $retval .= ' ' x $prologlen; }
  return($retval . $text);
}

sub sh_create_arg
# stores a command-line option and its associated shell script code
{ my ($part, $names, $help, $code, $flags, $helpprep) = @_;
  # $flags & 1: don't use the help text; &2: enforce dvhelp
  my $names_for_help = '';
  my $names_for_code = '';
  foreach my $one (split(/\|/, $names))
  { my $v;
    if ($one =~ m/^([^\=]+)\=(.+)$/)
    { $one = $1; $v = $2; $need_sh_arg_assign = 1; $need_sh_arg_dvhelp = 1; }
    else { $v = ''; }
    if ($one !~ m/^\-/) { $one = '--' . $one; }

    if ($names_for_help ne '') { $names_for_help .= ', '; }
    $names_for_help .= $one;
    if ($v ne '') { $names_for_help .= '=' . $v; }

    if ($names_for_code ne '') { $names_for_code .= ' | '; }
    $names_for_code .= $one;
    if ($v ne '') { $names_for_code .= '=*'; }
  }
  if (!($flags & 1))
  { push(@{$sh_arg[$part]}, [ $names_for_help, $help ]);
    if ($helpprep ne '') { push(@helpprepare, $helpprep); }
  }
  if ($flags & 2) { $need_sh_arg_dvhelp = 1; }
  $sh[SH_PART_ARG] .= "\n  $names_for_code)\n$code\n  ;;\n";
}

sub sh_end
# finishes everything and writes the configure script
{ if ($might_install_something) { &emit_sh(SH_PART_INIT, 'ic_installs='); }

  if ($might_install_programs)
  { &emit_sh(SH_PART_INIT, 'ic_program_prefix=', 'ic_program_suffix=');
    &sh_create_arg(SH_ARGPART_INSTALL, 'program-prefix=...',
      'standard program prefix [NONE]',
      '  ic_program_prefix="$i_aval"', '',
      'if test -z "$ic_program_prefix"; then ic_program_prefix_h=NONE; else ic_program_prefix_h="$ic_program_prefix"; fi');
    &sh_create_arg(SH_ARGPART_INSTALL, 'program-suffix=...',
      'standard program suffix [NONE]',
      '  ic_program_suffix="$i_aval"', '',
      'if test -z "$ic_program_suffix"; then ic_program_suffix_h=NONE; else ic_program_suffix_h="$ic_program_suffix"; fi');
  }

  if ($might_make_binaries) { &emit_sh(SH_PART_INIT, 'ic_binaries='); }

  if ($is_slang_needed{'c'})
  { &emit_sh(SH_PART_INIT, 'ic_CC=${CC-gcc}', 'ic_CFLAGS0="${CFLAGS-}"',
      'ic_CFLAGS="$ic_CFLAGS0"', 'ic_CPPFLAGS0="${CPPFLAGS-}"',
      'if test "x$ic_CC" = "xgcc"; then', '  ic_CFLAGS="-O2 $ic_CFLAGS"', 'fi',
      'if test "${LDFLAGS+set}" = set; then', '  ic_LDFLAGS="$LDFLAGS"',
      'else', '  ic_LDFLAGS=\'-s\'', 'fi', 'ic_objext="${OBJEXT-o}"');
    # CHECKME: don't use gcc as default C compiler, instead try
    # "for compiler in gcc cc c89 c99 ...."?
  }

  if ($might_use_infiles) { &emit_sh(SH_PART_INIT, 'ic_infiles='); }

  foreach my $one (@installer_subst)
  { my ($srcvar, $destvar, $behavior) = @{$one};
    &emit_sh(SH_PART_CODE, "eval pfinal0=\"\$$srcvar\"",
      'eval pfinal="$pfinal0"');
    &cop_substitute($destvar, '$pfinal', $behavior, 1);
  }

  # IMPLEMENTME: if ($#installer_magic < 0) { strip all installer magic
  # assignments from $sh[SH_PART_CODE] }

  &sh_create_arg(SH_ARGPART_GENERAL, 'verbose', 'print more information about what happens',
    "  ic_show_info=echo");
  &sh_create_arg(SH_ARGPART_GENERAL, 'quiet', 'don\'t print any messages',
    "  ic_fatal_error=false\n  ic_show_message=false");

  &sh_create_arg(SH_ARGPART_GENERAL, 'version|-V',
    'print the version information and exit', "  i_premature_exit=b\n  break");

  &sh_create_arg(SH_ARGPART_GENERAL, 'report', 'print a little configuration report and exit',
    "  i_premature_exit=c\n  break");

  if ($debug & 8)
  { &sh_create_arg(SH_ARGPART_GENERAL, 'log', 'enable logging to configure.log [DISABLED]', '  ic_log=echo', 2);
  }

  if ($debug & 16)
  { &sh_create_arg(SH_ARGPART_GENERAL, 'debug', 'enable configuration debugging [DISABLED]',
     "  ic_do_debug=:", 2);
  }

  &sh_create_arg(SH_ARGPART_GENERAL, 'help', 'print this help and exit',
    "  i_premature_exit=d\n  break");

  if ( (!$is_subconf) and ($sh_dist_collector ne '') )
  { &sh_create_arg(SH_ARGPART_GENERAL, 'dist*',
      'generate distribution archives and exit',
      "  ic_dist_arg=\"\$i_arg\"\n  i_premature_exit=e\n  break");
  }

  &write_configure_script();
}

sub cop_end
{ &deny_subconf('IC_END');
  if (!$found_ic_begin) { &fatal_error('missing IC_BEGIN'); }
  if ($found_ic_end) { &fatal_error('found duplicate IC_END'); }
  $found_ic_end = 1;
  &sh_end();
}

sub cop_ensure_launchdir
{ my $filename = $_[0];
  $ensure_launchdir = '
if test -f "' . $filename . '"; then :; else
' . &sh_fatal_error('configuration script launched from wrong directory!',
      '  ') . '
fi

';
}

sub emit_sh
# adds some shell script code to a certain part of the configure script
{ my ($part, @text) = @_;
  foreach my $one (@text) { $sh[$part] .= $one . "\n"; }
}

sub cop_emit_sh
{ my $text = $_[0];
  &emit_sh(SH_PART_CODE, $text);
}

sub sh_fatal_error
# returns shell script code to generate a fatal error
{ my ($text, $prettyprint) = @_;
  my $retval = $prettyprint . '$ic_fatal_error "$ic_me: ' . $text .
    "\" >&2\n" . $prettyprint . $sh_exit_failure;
  return($retval);
}

sub sh_log
# writes the given text to the log file (if logging is enabled)
{ my $retval;
  if ($debug & 8)
  { my ($text, $flags) = @_;
    $retval = '$ic_log "' . $text . '" >&' . $fd_sh_log;
    if ($flags & 2) { $retval .= "\n"; }
  }
  else { $retval = ''; }
  return($retval);
}

sub sh_dbgfd
{ if ($debug & 16)
  { $need_dbgfd = 1;
    return('$ic_dbgfd');
  }
  else { return($fd_sh_null); }
}

sub sh_debug
# returns shell script code to generate a debugging message;
# parameters: (text, flags, prettyprint)
{ my $retval;
  if ($debug & 16)
  { my ($text, $flags, $prettyprint) = @_;
    $retval = $prettyprint . '$ic_debug "' . $text . '" >&' . $fd_sh_log;
    if ($flags & 1) { $retval = "\n" . $retval; }
    if ($flags & 2) { $retval .= "\n"; }
  }
  else { $retval = ''; }
  return($retval);
}

sub cop_emit_config_h
{ my $text = $_[0];
  if ( ($package_config_h ne '') and ($text ne '') )
  { &emit_sh(SH_PART_CODE, 'cat >&' . $fd_sh_temp_config_h . ' <<EOF', $text,
      'EOF');
  }
}

sub is_variable_bad
# checks whether a user-defined shell script identifier looks okay
{ return($_[0] !~ m/^ICV\_/);
}

sub check_shellvar
{ my ($var, $macro) = @_;
  if (&is_variable_bad($var))
  { &fatal_error("$macro: a shell variable name does not begin with \"ICV_\"");
  }
}

sub sh_dist_subcall
{ my $relpath = $_[0];
  my $retval = '';
  foreach my $one (@subconf_dist)
  { $retval .= "(cd $one; ./configure --int-dist $one/$relpath \$ic_dist_arg)\n";
  }
  return($retval);
}

sub cop_dist
{ ($sh_dist_path, $sh_dist_collector, $sh_dist_compressor) = @_;
  if ($sh_dist_path !~ m/^[a-z]*$/)
  { &fatal_error('IC_DIST: the path may only consist of lower-case letters'); }
  if ($is_subconf)
  { if ($sh_dist_path ne '')
    { &fatal_error('IC_DIST: no path parameter allowed in sub-configuration');}
    if ($sh_dist_compressor ne '')
    { &fatal_error('IC_DIST: no compressor allowed in sub-configuration'); }
  }
}

sub cop_meta_require_version
{ my $wanted = $_[0];
  my $actual = $progversion;
  my @w = split(/\./, $wanted, 3);
  my @a = split(/\./, $actual, 3);

  if ( ($a[0] < $w[0]) or
       ( ($a[0] == $w[0]) and ($a[1] < $w[1]) ) )
  { too_old:
    &fatal_error("This version of itchconfig ($progversion) is too old. " .
      "$sourcefile requires at least itchconfig $wanted");
  }
  elsif ($a[1] == $w[1])
  { # We need more checks, but we have to be somewhat careful because future
    # version numbers (at least the "actual" ones) might be of the form
    # "0.0.0-foo" (where "-foo" is e.g. "-pre3"), not simply "0.0.0".
    my $a2 = $a[2];
    my $w2 = $w[2];
    if ($a2 =~ m/^([0-9]+)/) { $a2 = $1; }
    if ($w2 =~ m/^([0-9]+)/) { $w2 = $1; }
    if ($a2 < $w2) { goto too_old; }
  }
}

sub sh_headercomment
# adds a text to the beginning of "configure"
{ my $text = $_[0];
  if ($sh[SH_PART_HEADER] eq '') { $sh[SH_PART_HEADER] .= "#\n"; }
  $sh[SH_PART_HEADER] .= "# $text\n";
  # IMPLEMENTME: beautify!
}

sub sh_versioncomment
# adds a text to the output of "./configure --version"
{ my $text = $_[0];
  $sh[SH_PART_VERSION] .= "$_[0]\n";
  # IMPLEMENTME: beautify!
}

sub cop_meta_credit_partial
{ my $text = $_[0];
  if ($text !~ m/^\s*$/)
  { $text = "Parts of this script are $text.";
    &sh_headercomment($text);
    &sh_versioncomment($text);
  }
}

sub cop_meta_revision
{ my $text = $_[0];
  if ($text !~ m/^\s*$/)
  { $text =~ s/[\"\$]//g;
    $text = "From $package_name $package_version $sourcefile $text";
    &sh_headercomment($text);
  }
}

sub cop_cache
# changes the caching level
{ $caching = $_[0] + 0;
}


# Section: Substitution

my $inside_substblock = 0;
my @substdata = ();
my @substfd = ($fd_sh_temp_subst, $fd_sh_temp_makefile, $fd_sh_temp_config_h);

sub sh_substitute_flush
{ for (my $i = 0; $i <= 2; $i++)
  { if ($substdata[$i] ne '')
    { &emit_sh(SH_PART_CODE, 'cat >&' . $substfd[$i] . ' <<EOF',
        $substdata[$i] . 'EOF');
    }
  }
  @substdata = ();
}

sub sh_substitute
{ my ($id, $value, $behavior, $internal_flags) = @_;
  if ($behavior eq '') { $behavior = $substopt; }
  if ($behavior & 1)
  { my $e = ( ($internal_flags & 4) ? 'eval ' : '' );
    &emit_sh(SH_PART_CODE, "$e$id=\"$value\"");
  }
  if ($behavior & 2)
  { $substdata[0] .= 's,@' . $id . '@,' . $value . ',g' . "\n"; }
  if ($behavior & 4) { $substdata[1] .= "$id=$value\n"; }
  if ($behavior & 8)
  { my $v = $value;
    if ( (!($internal_flags & 2)) and ($v !~ m/^(\-)?[0-9]+$/) and
         (! (($v =~ m/^\"/) and ($v =~ m/\"$/)) ))
    { $v = '"' . $v . '"'; } # need quotation marks
    $substdata[2] .= "#define $id $v\n";
  }
  if ( ($internal_flags & 8) and (!($behavior & (1 | 2 | 4 | 8))) )
  { &emit_sh(SH_PART_CODE, ':'); }
# if ($internal_flags ne '') { $inside_substblock = 0; } # break the block
  if (!$inside_substblock) { &sh_substitute_flush(); }
}

sub cop_substitute
{ my ($id, $value, $behavior, $internal_flags) = @_;
  # $internal_flags: "&1": no identifier check for behavior 1; "&2": no
  # quotation for behavior 8; "&4": eval for behavior 1; "&8": make sure that
  # _something_ will be written to the configure script
  if ($behavior eq '') { $behavior = $substopt; }
  else
  { $behavior += 0;
    if ($package_makefile eq '') { $behavior &= ~4; }
    if ($package_config_h eq '') { $behavior &= ~8; }
    if ( (!($internal_flags & 1)) and ($behavior & 1) and
         (&is_variable_bad($id)) )
    { my $x = ( ($current_macro ne '') ? $current_macro : 'IC_SUBSTITUTE' );
      &fatal_error("$x: behavior & 1, but id (\"$id\") does not begin with \"ICV_\"");
    }
  }
  &sh_substitute($id, $value, $behavior, $internal_flags);
}

sub cop_substitute_output
{ my ($command, $id, $behavior, $flags) = @_;
  if ($flags eq '') { $flags = 1; }

  # if (flags & 4), we can't simply write foo=`....` because some shells don't
  # handle the exit code right then. So we do it the hard way...

  $command .= ' >&' . ( ($flags & 1) ? $fd_sh_temp_temp : $fd_sh_null ) .
    ' 2>&' . ( ($flags & 2) ? $fd_sh_temp_temp : $fd_sh_null );

  &emit_sh(SH_PART_CODE, 'exec ' . $fd_sh_temp_temp . '>$ic_tempdir/temp',
    $command);
  if ($flags & 4)
  { &emit_sh(SH_PART_CODE, 'i_x=$?');
    &cop_substitute($id . $idfoofix{'exitcode-id'}, '$i_x', $behavior, 2);
  }
  &emit_sh(SH_PART_CODE, &sh_close_fd($fd_sh_temp_temp, 1),
    'i_x="`cat $ic_tempdir/temp 2>&' . $fd_sh_null . '`"');
  &cop_substitute($id, '$i_x', $behavior);

  if ($command =~ m/^([A-Za-z0-9\_\-\.\~]*config\.guess)/)
  { $config_guess = $1; }
}

sub cop_substitute_begin
{ $inside_substblock = 1;
}

sub cop_substitute_end
{ $inside_substblock = 0;
  &sh_substitute_flush();
}

sub cop_substitute_behavior
{ my $behavior = $_[0];
  if ($package_makefile eq '') { $behavior &= ~4; }
  if ($package_config_h eq '') { $behavior &= ~8; }
  $substopt = $behavior;
}

sub cop_meta_substitute_prefix
{ my ($kinds, $foofix) = @_;
  if ( ($foofix eq '') or ($foofix =~ m/^[0-9]/) )
  { &fatal_error('IC_META_SUBSTITUTE_PREFIX: bad pre-/suffix parameter'); }
  my @k = &split_any('IC_META_SUBSTITUTE_PREFIX', $kinds, 'kinds');
  foreach my $one (@k)
  { if ($idfoofix{$one} eq '')
    { &fatal_error('IC_META_SUBSTITUTE_PREFIX: unknown kind "' . $one . '"'); }
    $idfoofix{$one} = $foofix;
  }
}


# Section: Binaries

sub split_any
{ my ($macro_name, $x, $what) = @_;
  $x =~ s/\n+/\ /g; # extra comfort ;-)
  my @xl = split(/\s+/, $x);
  if ($#xl < 0) { &fatal_error("$macro_name: no $what given"); }
  return(@xl);
}

sub split_binaries
{ return(&split_any($_[0], $_[1], 'binaries'));
}

sub need_slang
# marks a source-code language as needed
{ my ($macro_name, $slang) = @_;
  if (!$is_slang_known{$slang})
  { &fatal_error("$macro_name: unknown source-code language \"$slang\""); }
  $is_slang_needed{$slang} = 1;
}

sub cop_binary_create
{ my $binaries = $_[0];
  my @bin = &split_binaries('IC_BINARY_CREATE', $binaries);
  $might_make_binaries = 1;
  my $code = 'ic_binaries="$ic_binaries ' . $binaries . "\"\n";
  foreach my $b (@bin)
  { $code .= 'ic_binary_' . $b . "_libs=\n";
    foreach my $l (sort keys %is_slang_known)
    { # IMPLEMENTME: consider only the languages which are actually needed for
      # these binaries!
      $code .= 'ic_binary_' . $b . '_' . $l . "_sources=\n";
      if ($l eq 'c')
      { $code .= 'ic_binary_' . $b . '_cflags="$ic_CFLAGS"' . "\n"; }
    }
    push(@mf_phony, "binary-$b");
    push(@mf_clean, $b);
  }
  &emit_sh(SH_PART_CODE, $code);
}

sub cop_binary_add_source
{ my ($binaries, $language, $sourcefiles, $prefix) = @_;
  my @bin = &split_binaries('IC_SOURCEFILE_ADD', $binaries);

  if ($language eq '') { $language = 'c'; }
  else { $language = lc($language); }
  &need_slang('IC_BINARY_ADD_SOURCE', $language);

  my @source = split(/\s+/, $sourcefiles);
  my @finalsource = ();
  my @inf = ();
  if ( ($prefix ne '') and ($prefix !~ m/\/$/) ) { $prefix .= '/'; }
  foreach my $s (@source)
  { if ($s =~ m/^([0-9]+)\;(.+)$/)
    { my $count = $1 + 0;
      $s = $2;
      $might_use_infiles = 1;
      my $infile = $s;
      while ($count-- > 0) { $infile .= $infile_suffix; }
      push(@inf, $prefix . $infile);
    }
    push(@finalsource, $prefix . $s);
  }
  if ($#inf >= 0) { &sh_infile_add(@inf); }

  $sourcefiles = join(' ', @finalsource);
  foreach my $b (@bin)
  { my $id = 'ic_binary_' . $b . '_' . $language . '_sources';
    &emit_sh(SH_PART_CODE, "$id=\"\$$id $sourcefiles\"");
  }

  if ( ($package_makefile ne '') and ($language eq 'c') )
  { foreach my $one (@finalsource)
    { if ($one =~ s/\.c$/\./) { push(@mf_mostlyclean, $one); }
    }
  }
}

sub cop_binary_add_library
{ my ($binaries, $libraries) = @_;
  my @bin = &split_binaries('IC_LIBRARY_ADD', $binaries);
  foreach my $b (@bin)
  { my $id = 'ic_binary_' . $b . '_libs';
    &emit_sh(SH_PART_CODE, "$id=\"\$$id $libraries\"");
  }
}

sub cop_binary_reentrant
{ my $binaries = $_[0];
  my @bin = &split_binaries('IC_BINARY_REENTRANT', $binaries);
  foreach my $b (@bin)
  { my $id = 'ic_binary_' . $b . '_cflags';
    &emit_sh(SH_PART_CODE, "$id=\"\$$id -D_REENTRANT -D_THREAD_SAFE\"");
  }
}

sub cop_binary_use_gtk
{ my $binaries = $_[0];
  my @bin = &split_binaries('IC_BINARY_USE_GTK', $binaries);
  foreach my $b (@bin)
  { my $id0 = 'ic_binary_' . $b . '_';
    my $idc = $id0 . 'cflags'; my $idl = $id0 . 'libs';
    &emit_sh(SH_PART_CODE, "$idc=\"\$$idc `gtk-config --cflags`\"",
      "$idl=\"\$$idl `gtk-config --libs`\"");
  }
}

sub cop_binary_permission
{ my $binaries = $_[0];
  my @bin = &split_binaries('IC_BINARY_PERMISSION', $binaries);
  # IMPLEMENTME!
}


# Section: Installation

sub installtarget2shellvar
{ return('ic_path_' . &convname($_[0], 1));
}

sub installprefix
{ my $path = '';
  my $helppath = '';
  if ($_[0]) # it's a target for architecture-dependent stuff
  { $path = '${ic_path_archprefix}';
    $helppath = 'ARCHPREFIX';
    if (!$emitted_install_archprefix)
    { $emitted_install_archprefix = 1;
      &emit_sh(SH_PART_INIT, 'ic_path_archprefix=\'${ic_path_prefix}\'');
      &sh_create_arg(SH_ARGPART_INSTALLPATH, "path-archprefix=PATH",
        'path prefix for architecture-dependent files [PREFIX]',
        '  ic_path_archprefix="$i_aval"');
      push(@sh_require_abspath, 'ic_path_archprefix');
      push(@helpprepare_installpath, 'ic_path_archprefix');
    }
    goto emit_install_prefix;
  }
  else
  { $path = '${ic_path_prefix}';
    $helppath = 'PREFIX';
    emit_install_prefix:
    if (!$emitted_install_prefix)
    { $emitted_install_prefix = 1;
      &emit_sh(SH_PART_INIT, 'ic_path_prefix=/usr/local');
      &sh_create_arg(SH_ARGPART_INSTALLPATH, "path-prefix=PATH",
        'path prefix for architecture-independent files [/usr/local]',
        '  ic_path_prefix="$i_aval"');
      push(@sh_require_abspath, 'ic_path_prefix');
      push(@helpprepare_installpath, 'ic_path_prefix');
    }
  }
  return( ($path, $helppath) );
}

sub cop_install_create
{ my ($target, $path, $files, $help, $flags, $behavior, $magic_code) = @_;
  &deny_subconf('IC_INSTALL_CREATE');
  if ($target eq '') { &fatal_error('IC_INSTALL_CREATE: missing target name');}
  if ($target !~ m/^[A-Za-z0-9\_]+$/)
  { &fatal_error("IC_INSTALL_CREATE: bad target name \"$target\""); }

  $might_install_something = 1;
  my $target_conv = &installtarget2shellvar($target);
  my $helppath = '';
  my $pr = $installer_data{$target};
  if (!defined($pr)) { goto nonstandard; }

  my ($stdarchdep, $stdpath, $stdhelp, $stdflags, $stdmagic) = @{$pr};
  ($path, $helppath) = &installprefix($stdarchdep);
  if ($stdpath ne '') { $path .= "/$stdpath"; $helppath .= "/$stdpath"; }
  $path .= "/$target"; $helppath .= "/$target";
  if ($help eq '') { $help = $stdhelp; }
  if ($flags ne '') { $flags &= ~1; }
  elsif ($stdflags ne '') { $flags = $stdflags + 0; }
  if ($magic_code eq '') { $magic_code = $stdmagic; }
  push(@sh_require_abspath, $target_conv);
  goto finish;

  nonstandard:
  if ($flags & 1)
  { $path = ''; $helppath = '(auto-detected)';
    push(@sh_require_abspath2, $target_conv);
  }
  else
  { if ($path =~ m/^\//) { $helppath = $path; }
    else
    { my $p0;
      my $f0 = ( ($flags & 8) ? 1 : 0 );
      ($p0, $helppath) = &installprefix($f0);
      if ($path eq '') { $path = $target; }
      $helppath .= "/$path"; $path = "$p0/$path";
    }
    push(@sh_require_abspath, $target_conv);
  }

  finish:
  push(@act_arg, ["path-$target", $help, $target_conv]);
  if ($help ne '') { $help .= ' '; }
  $help .= "[$helppath]";
#  if ($helppath eq '(auto-detected)') { $help .= "[$helppath]"; }
#  else { $help .= "[\$" . $target_conv . "_h]"; }
  &emit_sh(SH_PART_INIT, "$target_conv='$path'");
  &sh_create_arg(SH_ARGPART_INSTALLPATH, "path-$target=PATH", $help,
    "  $target_conv=\"\$i_aval\"");
  if (!($flags & 1)) { push(@helpprepare_installpath, $target_conv); }
  &emit_sh(SH_PART_CODE, "ic_installs=\"\$ic_installs $target\"");
  push(@mf_phony, "install-$target");

  my $qf = 'ic_install_' . $target . '_files=';
  if ($files ne '')
  { $qf .= '"' . $files . '"';
    # FIXME: if $sop and the files are binaries, their names probably must
    # be .= $sh_progext!
  }
  &emit_sh(SH_PART_CODE, $qf);

  my $permx = ( ($flags & (2 | 4)) ? 'x' : '' );
  my $perm = "u=rw$permx,g=r$permx,o=r$permx";
  &emit_sh(SH_PART_CODE, 'ic_install_' . $target . '_perm="' . $perm . '"');

  my $qm = 'ic_install_' . $target . '_magic=m';
  if ( ($magic_code eq '') and ($flags & 2) )
  { $magic_code = '%F%="$%PP%$%F%$%PS%%PE%"'; } # default magic for programs
  if ($magic_code eq '_') { $magic_code = ''; } # disable any default magic
  if ($magic_code eq '') { $qm .= '0'; }
  else
  { $magic_code =~ s/\%F\%/file/g;
    $magic_code =~ s/\%P\%/path/g;
    $magic_code =~ s/\%PE\%/$sh_progext/g;
    $magic_code =~ s/\%PN\%/$package_name/g;
    if ($flags & 2)
    { $magic_code =~ s/\%PP\%/ic_program_prefix/g;
      $magic_code =~ s/\%PS\%/ic_program_suffix/g;
    }
    $magic_code =~ s/\%PV\%/$package_version/g;
    &debug("Installer magic for $target: *$magic_code*");

    # check whether we already use this magic for another installation target
    # in order to avoid wasteful duplication
    for (my $one = 0; $one <= $#installer_magic; $one++)
    { if ($installer_magic[$one] eq $magic_code)
      { $qm .= ($one + 1); goto magic_finish; }
    }

    push(@installer_magic, $magic_code);
    $qm .= ($#installer_magic + 1);
  }
  magic_finish:
  &emit_sh(SH_PART_CODE, $qm);

  if ($flags & 2) { $might_install_programs = 1; }
  if ($behavior != 0)
  { push(@installer_subst, [ $target_conv, $idfoofix{'path-install'} .
      &convname($target), $behavior ]);
  }
}

sub sh_install_add_file
{ my ($target, $files) = @_;
  my $tf = 'ic_install_' . $target . '_files';
  &emit_sh(SH_PART_CODE, "$tf=\"\$$tf $files\"");
}

sub cop_install_add_file
{ my ($targets, $files) = @_;
  my @targ = split(/\s+/, $targets);
  foreach my $t (@targ) { &sh_install_add_file($t, $files); }
}

sub cop_install_path_set
{ my ($target, $code) = @_;
  my $target_conv = &installtarget2shellvar($target);
  &emit_sh(SH_PART_CODE, $target_conv . '=' . $code);
}


# Section: Package Options

sub option2ids
# converts a basic feature/option name to internal and external identifiers
{ my ($name, $idpref) = @_;
  my $conv = &convname($name);
  my $intid = 'ic_option_' . &convname($name, 1); # for the configure script
  my $extid = $idfoofix{$idpref} . &convname($name); # e.g. for config.h
  return( ($intid, $extid) );
}

sub cop_option_enable
{ my ($name, $value, $help, $behavior) = @_;
  &deny_subconf('IC_OPTION_ENABLE');
  my ($intid, $extid) = &option2ids($name, 'option-enable');
  if ($value != 1) { $value = 0; }
  if ($help ne '') { $help .= ' '; }
  $help .= '[' . ( ($value == 1) ? 'EN' : 'DIS' ) . 'ABLED]';
  $sh[SH_PART_INIT] .= "$intid=$value\n";
  &sh_create_arg(SH_ARGPART_PACKAGE, "enable-$name", $help, "  $intid=1", 2);
  &sh_create_arg(SH_ARGPART_PACKAGE, "disable-$name", $help, "  $intid=0", 3);
  $need_sh_arg_disable = 1;
  &cop_substitute($extid, "\$$intid", $behavior, 1 | 2);
}

sub cop_option_set
{ my ($name, $value, $help, $behavior) = @_;
  &deny_subconf('IC_OPTION_SET');
  my ($intid, $extid) = &option2ids($name, 'option-set');
  if ($value eq '') { $value = 0; }
  if ($help ne '') { $help .= ' '; }
  $help .= "[$value]";
  &emit_sh(SH_PART_INIT, "$intid=$value");
  &sh_create_arg(SH_ARGPART_PACKAGE, "set-$name=VALUE", $help,
    "  $intid=\"\$i_aval\"");
  &cop_substitute($extid, "\$$intid", $behavior, 1 | 2);
}


# Section: Makefiles

sub split_targets
{ return(&split_any($_[0], $_[1], 'targets'));
}

sub sh_makefile_target_add_code
{ my ($macro_name, $targets, $code) = @_;
  my @t = &split_targets($macro_name, $targets);
  $code = &escape_here_twice($code);
  foreach my $one (@t)
  { &emit_sh(SH_PART_CODE, 'cat >>/$ic_tempdir/target_' . $one . ' <<EOF',
      $code, 'EOF');
  }
}

sub cop_makefile_target_create
{ my ($target, $flags, $dep, $code) = @_;
  $might_create_extra_targets = 1;
  &emit_sh(SH_PART_CODE, "ic_extra_targets=\"\$ic_extra_targets $target\"");
  my $qd = 'ic_extra_target_' . $target . '_dep=';
  if ($dep ne '') { $qd .= "\"$dep\""; }
  # create a temporary file for the target:
  &emit_sh(SH_PART_CODE, $qd, "exec $fd_sh_temp_temp>/\$ic_tempdir/target_$target",
    &sh_close_fd($fd_sh_temp_temp, 1));
  if ($code ne '')
  { &sh_makefile_target_add_code('IC_TARGET_CREATE', $target, $code); }
  if (!($flags & 1)) { push(@mf_phony, $target); }
}

sub cop_makefile_target_add_dep
{ my ($targets, $dep) = @_;
  my @t = &split_targets('IC_MAKEFILE_TARGET_ADD_DEP', $targets);
  foreach my $one (@t)
  { my $q = 'ic_extra_target_' . $one . '_dep';
    &emit_sh(SH_PART_CODE, "$q=\"\$$q $dep\"");
  }
}

sub cop_makefile_target_add_code
{ my ($targets, $code) = @_;
  &sh_makefile_target_add_code('IC_MAKEFILE_TARGET_ADD_CODE', $targets, $code);
}

sub cop_makefile_target_default_set
{ $might_change_default_target = 1;
  &emit_sh(SH_PART_CODE, 'ic_default_target="' . $_[0] . '"');
}

sub cop_makefile_preserve
{ $preserve_makefile = 1;
}


# Section: .in Files

sub sh_infile_add
{ my @files = @_;
  # We make sure that any .in file is added at most once, so that it won't be
  # converted several times unnecessarily.
  foreach my $one (@files)
  { my $qo = quotemeta($one); # FIXME: quotemeta() quotes too much for us!
    &emit_sh(SH_PART_CODE, "if expr \"\$ic_infiles\" : \"$qo\" >&$fd_sh_null 2>&1; then :; else", "  ic_infiles=\"\$ic_infiles $one\"", 'fi');
    while ($one =~ s/$infile_suffix_re$//) { push(@mf_distclean, $one); }
  }
}

sub cop_infile_add
{ $might_use_infiles = 1;
  &sh_infile_add(split(/\s+/, $_[0]));
}

sub set_infile_suffix
{ $infile_suffix = $_[0];
  if ( ($infile_suffix eq '') or (($infile_suffix =~ tr/A-Za-z0-9\.//c) > 0) )
  { &fatal_error('IC_INFILE_SUFFIX_SET: illegal suffix'); }
  $infile_suffix_re_sh = $infile_suffix;
  $infile_suffix_re_sh =~ s/\./\\\./g;
  $infile_suffix_re = $infile_suffix_re_sh; # CHECKME!
}

sub cop_infile_suffix_set
{ &set_infile_suffix($_[0]);
}


# Section: Paths

sub split_programs
{ return(&split_any($_[0], $_[1], 'program names'));
}

sub cop_path_program
{ my ($program, $id, $behavior, $variants) = @_;
  $need_convpath = 1;
  if ($id eq '') { $id = $idfoofix{'path-program'} . &convname($program); }
  if ($variants eq '_') # try to use a default variants list
  { my %V = ('awk' => 'gawk mawk nawk awk', 'perl' => 'perl perl5',
      'tar' => 'gtar gnutar tar');
    $variants = $V{$program};
  }
  if ($variants ne '')
  { # Make sure that $program is one of the variants:
    @v = split(/\s+/, $variants);
    my $found = 0;
    foreach my $one (@v) { if ($one eq $program) { $found = 1; last; } }
    if (!$found) { unshift(@v, $program); }
    $variants = join(' ', @v);
  }
  my $do_v = ( ($variants ne '') ? 1 : 0 );
  my $brk = '    break' . ($do_v ? ' 2' : '');

  &emit_sh(SH_PART_CODE, '', '# Path of program "' . $program . '"');
  &cop_message_checking("the path of the program \\\"$program\\\"");
  &emit_sh(SH_PART_CODE, 'i_pfinal=');
  if ($do_v) { &emit_sh(SH_PART_CODE, 'for i_var in ' . $variants, 'do'); }
  &emit_sh(SH_PART_CODE, 'for i_pp in $ic_convpath', 'do',
    '  i_p0="$i_pp/' . ( ($do_v) ? '$i_var' : $program ) . $sh_progext . '"',
    '  if test -f "$i_p0"; then', '    i_pfinal="$i_p0"', $brk, '  fi','done');
  if ($do_v) { &emit_sh(SH_PART_CODE, 'done'); }
  &emit_sh(SH_PART_CODE, 'if test -n "$i_pfinal"; then');
  &cop_substitute($id, '$i_pfinal', $behavior, 8);
  &emit_sh(SH_PART_CODE, 'else', '  i_pfinal=\'(NOT FOUND!)\'', 'fi');
  &cop_message_checkresult('$i_pfinal');
}

sub cop_path_programs
{ my ($programs, $behavior) = @_;
  my @p = &split_programs('IC_PATH_PROGRAMS', $programs);
  foreach my $one (@p) { &cop_path_program($one, '', $behavior); }
  # IMPLEMENTME: the loop should happen in configure, not in Perl...
}


# Section: Conditionals
# IMPROVEME: use "case ... esac" instead of "if test ... fi"!

sub cop_if_option_enabled
{ my $name = $_[0];
  my ($intid, $dummy) = &option2ids($name, 'option-enable');
  &emit_sh(SH_PART_CODE, "if test \"x\$$intid\" = x1; then");
}

sub cop_else_option_enabled
{ &emit_sh(SH_PART_CODE, 'else');
}

sub cop_endif_option_enabled
{ &emit_sh(SH_PART_CODE, 'fi');
}

sub cop_if_option_disabled
{ my $name = $_[0];
  my ($intid, $dummy) = &option2ids($name, 'option-enable');
  &emit_sh(SH_PART_CODE, "if test \"x\$$intid\" = x0; then");
}

sub cop_endif_option_disabled
{ &emit_sh(SH_PART_CODE, 'fi');
}

sub cop_if_identifier
{ my $id = $_[0];
  &check_shellvar($id, 'IC_IF_IDENTIFIER');
  &emit_sh(SH_PART_CODE, "if test \"x\$$id\" != x0; then");
}

sub cop_else_identifier
{ &emit_sh(SH_PART_CODE, 'else');
}

sub cop_endif_identifier
{ &emit_sh(SH_PART_CODE, 'fi');
}

sub cop_if_not_identifier
{ my $id = $_[0];
  &check_shellvar($id, 'IC_IF_NOT_IDENTIFIER');
  &emit_sh(SH_PART_CODE, "if test \"x\$$id\" = x0; then");
}

sub cop_endif_not_identifier
{ &emit_sh(SH_PART_CODE, 'fi');
}

sub cop_if_install_path_not_set
{ my $target = $_[0];
  my $target_conv = &installtarget2shellvar($target);
  &emit_sh(SH_PART_CODE, "if test -z \"\$$target_conv\"; then");
}

sub cop_endif_install_path_not_set
{ &emit_sh(SH_PART_CODE, 'fi');
}


# Section: Messages

sub cop_message_info
{ &emit_sh(SH_PART_CODE, "\$ic_show_info \"$_[0]\"");
}

sub cop_message_warning
{ &emit_sh(SH_PART_CODE, "\$ic_show_message \"Warning: $_[0]\"");
}

sub cop_message_error
{ &emit_sh(SH_PART_CODE, '{', &sh_fatal_error($_[0]), ':;', '}');
}

sub cop_message_checking
{ my ($text, $dest) = @_;
  if ($dest eq '') { $dest = SH_PART_CODE; }
  &emit_sh($dest,
    "\$ic_show_message \$ic_ECHO_N \"Checking $text... \$ic_ECHO_C\"");
  if ($debug & 8) { &emit_sh($dest, &sh_log("Checking $text...")); }
}

sub cop_message_checkresult
{ my ($text, $dest) = @_;
  if ($dest eq '') { $dest = SH_PART_CODE; }
  &emit_sh($dest, "\$ic_show_message \"\${ic_ECHO_T}$text\"");
  if ($debug & 8) { &emit_sh($dest, &sh_log("Check result: $text")); }
}

sub cop_message_debug
{ &sh_debug($_[0], 3);
}


# Section: C Tests

sub c_definc
{ $need_c_definc = 1;
  return('__IC_C_DEFINC__');
}

sub c_check_headers
# collects names of C headers which have to be checked for _internal_ reasons
{ my @headers = @_;
  foreach my $one (@headers) { $check_c_headers{$one} = 1; }
}

sub sh_c_store
{ my ($resultid, $behavior, $flags, $internal_flags) = @_;
  # $internal_flags: "&1": don't check $resultid identifier
  if ($resultid ne '')
  { my $csflags;
    if ($internal_flags & 1) { $csflags = 1; }
    else { $csflags = 0; }
    &cop_substitute($resultid, '$i_result', $behavior, 2 | $csflags);
    if ($flags & 2)
    { &cop_substitute($resultid . $idfoofix{'exitcode-id'}, '$i_result_e',
        $behavior, 2 | $csflags);
    }
    if ($flags & 4)
    { &cop_substitute($resultid . $idfoofix{'output-id'}, '$i_result_o',
        $behavior, $csflags);
    }
  }
}

sub sh_c_write_testfile
# writes some C code to testfile.c
{ my ($code, $dest) = @_;
  if ($dest eq '') { $dest = SH_PART_CODE; }
  &emit_sh($dest, 'cat >$ic_testfile.c <<__IC_EOF__', $code,
    '__IC_EOF__');
}

sub sh_c_build
# compiles/links/runs some C source code
{ my ($step, $resultid, $behavior, $flags, $libs, $code, $internal_flags)= @_;
  # $step: 1=compile, 2=link, 3=run
  # $internal_flags: "&1": don't here-escape $code; "&2": don't substitute
  # result variables (i_result, i_result_e, i_result_o); "&4": don't check
  # the $resultid identifier; "&8": use SH_PART_SLANG, not SH_PART_CODE
  my $dest = ( ($internal_flags & 8) ? SH_PART_SLANG : SH_PART_CODE );
  &need_slang('(DUMMY)', 'c');
  if (!($internal_flags & 1)) { $code = &escape_here($code); }
  $code =~ s/\_\_IC\_C\_DEFINC\_\_/\$ic_c_definc/g;
  &sh_c_write_testfile($code, $dest);

  if ($debug & 16)
  { my @steps = ('', 'compile', 'link', 'run');
    my $s = $steps[$step];
    if ($s eq '') { $s = '(INTERNAL ERROR)'; } # "should not happen"
    &emit_sh($dest, &sh_debug('Trying to ' . $s . ' this C code:'),
      '$ic_do_debug && cat <$ic_testfile.c >&' . $fd_sh_log,
      &sh_debug('(End of C code)'), 'i_x=NONE');
  }

  &emit_sh($dest, 'i_result=0'); # assume failure
  if ($flags & 2) { &emit_sh($dest, 'i_result_e=1'); }
  if ($flags & (4 | 8)) { &emit_sh($dest, 'i_result_o='); }

  # The -o switch is _always_ needed; otherwise a compiler might put the output
  # file into the current directory instead of the temporary directory...
  my $o = ' -o $ic_testfile.$ic_objext';
# my $o = ( ($step >= 2) ? (' -o $ic_testfile.$ic_objext') : ('') );

  &emit_sh($dest, 'if $ic_CC $ic_CFLAGS0 $ic_CPPFLAGS0 -c $ic_testfile.c'
    . $o . ' >&' . $fd_sh_null . ' 2>&' . &sh_dbgfd(). '; then');
  if ($step >= 2)
  { $libs =~ s/^\s+//; $libs =~ s/\s+$//;
    if ($libs ne '')
    { if ($libs =~ m/^\$/) # it's a shell variable (e.g. IC_C_FUNCTIONS_OPTLIB)
      { &emit_sh($dest, "  case $libs in", "    '') i_realib='' ;;",
          "    *) i_realib=\"-l$libs\" ;;", '  esac');
        $libs=' $i_realib';
      }
      else
      { $libs = ' ' . $libs; $libs =~ s/\s+/\ \-l/g; $libs = ' ' . $libs;
      }
    }
    &emit_sh($dest, '  if $ic_CC $ic_CFLAGS0 $ic_CPPFLAGS0 $ic_LDFLAGS $ic_testfile.$ic_objext $LIBS' . $libs . ' -o $ic_testfile >&' . $fd_sh_null . ' 2>&' . &sh_dbgfd() . '; then',
      '    i_result=1');
    if ( ($step >= 3) and ($flags & (1 | 2 | 4 | 8)) )
    { # We actually have to run the program. If e.g. ($flags & (2|4)), we can't
      # simply write "i_result_o=`$ic_testfile ...`" and "i_result_e=$?" here
      # because then i_result_e might be set to the wrong value - $? isn't
      # always set correctly by some shells in `...` assignments. So we do it
      # like in cop_substitute_output().
      my $command = '$ic_testfile >&' .
        ( ($flags & 4) ? $fd_sh_temp_temp : $fd_sh_null ) . ' 2>&' .
        ( ($flags & 8) ? $fd_sh_temp_temp : $fd_sh_null );
      if ($flags & (4 | 8))
      { &emit_sh($dest, '    exec ' . $fd_sh_temp_temp .
          '>$ic_tempdir/temp');
      }
      &emit_sh($dest, "    $command", '    i_x=$?');
      if ($flags & (4 | 8))
      { &emit_sh($dest, &sh_close_fd($fd_sh_temp_temp, 1, '    ')); }
      if ($flags & 1)
      { &emit_sh($dest, '    if test "x$i_x" != "x0"; then',
          '      i_result=0', '    fi');
      }
      if ($flags & 2) { &emit_sh($dest, '    i_result_e=$i_x'); }
      if ($flags & (4 | 8))
      { &emit_sh($dest,
          '    i_result_o="`cat $ic_tempdir/temp 2>&' . $fd_sh_null . '`"');
      }
    }
    &emit_sh($dest, '  fi'); # if linked
  }
  else { &emit_sh($dest, '  i_result=1'); }
  &emit_sh($dest, 'fi'); # if compiled

  if ($debug & 16)
  { my $res = 'result=$i_result, x=$i_x';
    if ($flags & 2) { $res .= ', e=$i_result_e'; }
    if ($flags & (4 | 8)) { $res .= ', o=$i_result_o'; }
    &emit_sh($dest, &sh_debug($res));
  }

  if (!($internal_flags & 2))
  { my $scsflags = ( ($internal_flags & 4) ? 1 : 0 );
    &sh_c_store($resultid, $behavior, $flags, $scsflags);
  }
}

sub cop_c_compile
{ my ($resultid, $behavior, $code, $internal_flags) = @_;
  &sh_c_build(1, $resultid, $behavior, 0, '', $code, $internal_flags);
}

sub cop_c_link
{ my ($resultid, $behavior, $libs, $code, $internal_flags) = @_;
  &sh_c_build(2, $resultid, $behavior, 0, $libs, $code, $internal_flags);
}

sub cop_c_run
{ my ($resultid, $behavior, $flags, $libs, $code, $internal_flags) = @_;
  &sh_c_build(3, $resultid, $behavior, $flags, $libs, $code, $internal_flags);
}

sub will_store_result__id
{ my ($macro, $resultid) = @_;
  if ( ($package_config_h eq '') and ($resultid eq '') )
  { &show_warning("$macro: test does not have any effect (no config.h file, no result id)");
    return(0);
  }
  else { return(1); }
}

sub will_store_result__behavior
{ my ($macro_name, $behavior) = @_;
  if ( ($package_config_h eq '') and ($behavior == 0) )
  { &show_warning("$macro_name: test does not have any effect (no config.h file, behavior is 0)");
    return(0);
  }
  else { return(1); }
}

sub macro_begin
# basically adds a tiny comment to "configure" for readability
{ my ($macro_name, $dest) = @_;
  if ($dest eq '') { $dest = SH_PART_CODE; }
  &emit_sh($dest, '', "# $macro_name", &sh_log("macro begin: $macro_name"));
  $current_macro = $macro_name;
}

sub macro_end
{ $current_macro = '';
}

sub macro_begin_c
{ my ($macro_name, $dest) = @_;
  &macro_begin($macro_name, $dest);
  &need_slang($macro_name, 'c');
}

sub cop_c_const
{ # C test code taken from autoconf-2.52/lib/autoconf/c.m4, AC_C_CONST
  my ($resultid, $behavior) = @_;
  if (!&will_store_result__id('IC_C_CONST', $resultid)) { return; }
  &macro_begin_c('IC_C_CONST');
  &cop_message_checking('whether the C const modifier works');
  &cop_c_compile($resultid, $behavior,
'int main()
{
#ifndef __cplusplus
  /* Ultrix mips cc rejects this. */
  typedef int charset[2];
  const charset x;
  /* SunOS 4.1.1 cc rejects this. */
  char const *const *ccp;
  char **p;
  /* NEC SVR4.0.2 mips cc rejects this. */
  struct point {int x, y;};
  static struct point const zero = {0,0};
  /* AIX XL C 1.02.0.0 rejects this. It does not let you subtract one const X*
     pointer from another in an arm of an if-expression whose if-part is not a
     constant expression. */
  const char *g = "string";
  ccp = &g + (g ? g-g : 0);
  /* HPUX 7.0 cc rejects these. */
  ++ccp;
  p = (char**) ccp;
  ccp = (char const *const *) p;
  { /* SCO 3.2v4 cc rejects this.  */
    char *t;
    char const *s = 0 ? (char *) 0 : (char const *) 0;
    *t++ = 0;
  }
  { /* Someone thinks the Sun supposedly-ANSI compiler will reject this.  */
    int x[] = {25, 17};
    const int *foo = &x[0];
    ++foo;
  }
  { /* Sun SC1.0 ANSI compiler rejects this -- but not the above. */
    typedef const int *iptr;
    iptr p = 0;
    ++p;
  }
  { /* AIX XL C 1.02.0.0 rejects this saying
       "k.c", line 2.27: 1506-025 (S) Operand must be a modifiable lvalue. */
    struct s { int j; const int *ap[3]; };
    struct s *b; b->j = 5;
  }
  { /* ULTRIX-32 V3.1 (Rev 9) vcc rejects this. */
    const int foo = 10;
  }
#endif
  return(0);
}');
  &emit_sh(SH_PART_CODE, 'if test "x$i_result" = "x0"; then');
  &cop_message_checkresult('no');
  &cop_emit_config_h('#define const');
  &emit_sh(SH_PART_CODE, 'else');
  &cop_message_checkresult('yes');
  &emit_sh(SH_PART_CODE, 'fi');
  &macro_end();
}

sub cop_c_inline
{ # C test algorithm similar to autoconf-2.52/lib/autoconf/c.m4, AC_C_INLINE
  my ($resultid, $behavior) = @_;
  if (!&will_store_result__id('IC_C_INLINE', $resultid)) { return; }
  &macro_begin_c('IC_C_INLINE');
  &cop_message_checking('whether the C inline modifier works');
  &emit_sh(SH_PART_CODE, 'working_inline=',
    'for keyword in inline __inline__ __inline', 'do');
  &cop_c_compile($resultid, $behavior,
'#ifndef __cplusplus
typedef int foo_type;
static $keyword foo_type static_foo_func() { return(0); }
$keyword foo_type foo_func() { return(0); }
#endif
int main()
{ return(0);
}', 1 | 2);
  &emit_sh(SH_PART_CODE, 'if test "x$i_result" = "x1"; then',
    '  working_inline="$keyword"', '  break', 'fi', 'done',
    'if test "$working_inline" != "inline"; then');
  &cop_emit_config_h('#define inline $working_inline'); # need to define it
  &emit_sh(SH_PART_CODE, 'fi');
  &cop_message_checkresult('$working_inline');
  &sh_c_store($resultid, $behavior, 0);
  &macro_end();
}

sub split_c_functions
{ return(&split_any($_[0], $_[1], 'C functions'));
}

sub sh_c_functions
{ # Test algorithm similar to autoconf-2.52/lib/autoconf/functions.m4,
  # AC_CHECK_FUNCS
  my ($macro_name, $functions, $behavior, $internal_optlib) = @_;
  my $o = $internal_optlib;
  my $p = 0; # "need the i_putsym stuff?"
  if ($o ne '')
  { my @f = &split_c_functions($macro_name, $functions);
    foreach my $one (@f)
    { if ( ($one eq '0') or ($one eq '1') ) { $p = 1; break; }
    }
  }
  if ($p) { &emit_sh(SH_PART_CODE, 'i_putsym=:'); }
  &emit_sh(SH_PART_CODE, 'for i_func in ' . $functions, 'do');
  if ($p)
  { # handle special control stuff
    &emit_sh(SH_PART_CODE, 'case $i_func in',
      '  0) i_putsym=false; continue ;;', '  1) i_putsym=:; continue ;;',
      'esac');
  }
  &emit_sh(SH_PART_CODE,
    'i_r=`echo "$i_func" | sed "y,$ic_az,$ic_AZ,;s,[^_${ic_Az09}],_,g"`',
    'i_resultid=\'' . $idfoofix{'have-c-function'} . '\'${i_r}');
  &cop_message_checking('whether' .
    ( ($o eq '') ? ' the C function' : '/where') . ' $i_func() exists');
    # (can't be that verbose if $o; would easily exceed 80 columns per line...)
  if ($o ne '') { &emit_sh(SH_PART_CODE, "for i_lib in '' $o", 'do'); }
  &cop_c_link('$i_resultid', $behavior, (($o eq '') ? '' : '$i_lib'),
    '#include "' . $ic_int_config_h .'"
#include <assert.h>
#ifdef __cplusplus
extern "C"
#endif
char $i_func();
char (*f)();

int main()
{
#if defined(__stub_$i_func) || defined(__stub___$i_func)
  choke me
#else
  f=$i_func;
#endif
  return(0);
}', 1 | 2);
  if ($o ne '')
  { &emit_sh(SH_PART_CODE, 'case $i_result in', '  1) break ;;', 'esac',
      'i_need_lib=:', 'done');
  }
  &emit_sh(SH_PART_CODE, 'case $i_result in');
  if ($o eq '') { &emit_sh(SH_PART_CODE, '  0) i_x=no ;;', '  *) i_x=yes ;;');}
  else
  { &emit_sh(SH_PART_CODE, '  0) i_x=no', '     i_need_lib=false', '     ;;',
      '  *) i_x=yes', '     case $i_lib in',
      '     \'\') i_x="$i_x (libc)" ;;',
      '     *) i_x="$i_x (-l$i_lib)" ;;', '     esac', '     ;;');
  }
  &emit_sh(SH_PART_CODE, 'esac');
  &cop_message_checkresult('$i_x');
  if ($p) { &emit_sh(SH_PART_CODE, 'if $i_putsym; then'); }
  &cop_substitute('$i_resultid', '$i_result', $behavior, 1 | 2 | 4);
  if ($p) { &emit_sh(SH_PART_CODE, 'fi'); }
  &emit_sh(SH_PART_CODE, 'done');
}

sub cop_c_functions
{ my ($functions, $behavior) = @_;
# my @f = &split_c_functions('IC_C_FUNCTIONS', $functions);
  &macro_begin_c('IC_C_FUNCTIONS');
  &sh_c_functions('IC_C_FUNCTIONS', $functions, $behavior);
  &macro_end();
}

sub cop_c_functions_optlib
{ my ($lib, $functions, $binaries, $behavior) = @_;
  if ($lib eq '') { &fatal_error('IC_C_FUNCTIONS_OPTLIB: no library given'); }
  &macro_begin_c('IC_C_FUNCTIONS_OPTLIB');
  &emit_sh(SH_PART_CODE, 'i_need_lib=false');
  &sh_c_functions('IC_C_FUNCTIONS_OPTLIB', $functions, $behavior, $lib);
  &emit_sh(SH_PART_CODE, 'if $i_need_lib; then', '  for i_bin in ' .
    ( ($binaries ne '') ? $binaries : '$ic_binaries' ), '  do',
    '    eval i_var="ic_binary_${i_bin}_libs"',
    '    eval i_val="$`echo $i_var`"',
    '    eval "$i_var=\'$i_val -l' . $lib . '\'"', '  done', 'fi');
  &macro_end();
}

sub cop_c_headers
{ # Test algorithm similar to autoconf-2.52/lib/autoconf/headers.m4,
  # AC_CHECK_HEADERS
  my ($headers, $behavior, $internal_flags) = @_;
  my $is_definc_init = ( ($internal_flags & 1) ? 1 : 0 );
  my $pref = ( ($internal_flags & 2) ? 'HAVE_' : $idfoofix{'have-c-header'} );
  my $cccflags = 1 | 2;
  if ($is_definc_init) { $cccflags |= 8; $behavior = ''; }
  my $dest = ( ($is_definc_init) ? SH_PART_SLANG : SH_PART_CODE );
  &macro_begin_c('IC_C_HEADERS', $dest);
  &emit_sh($dest, 'for i_header in ' . $headers, 'do',
    'i_r=`echo "$i_header" | sed "y,$ic_az,$ic_AZ,;s,[^_${ic_Az09}],_,g"`',
    'i_resultid=\'' . $pref . '\'${i_r}');
  &cop_message_checking('whether the C header file $i_header exists', $dest);
  # FIXME: we need an additional, cpp-related check here!
  &cop_c_compile('$i_resultid', $behavior, '#include <$i_header>', $cccflags);
  &emit_sh($dest, 'case $i_result in', '  0) i_x=no ;;',
    '  *) i_x=yes ;;', 'esac');
  &cop_message_checkresult('$i_x', $dest);
  if ($is_definc_init)
  { &emit_sh(SH_PART_SLANG,
      'echo "#define $i_resultid $i_result" >>$ic_int_config_h');
  }
  else
  { &cop_substitute('$i_resultid', '$i_result', $behavior, 1 | 2 | 4);
  }
  &emit_sh($dest, 'done');
  &macro_end();
}

sub cop_c_headers_stdc
{ # Test algorithm similar to autoconf-2.52/lib/autoconf/headers.m4,
  # AC_HEADER_STDC
  my ($resultid, $behavior, $internal_flags) = @_;
  if ($resultid eq '') { $resultid = 'STDC_HEADERS'; }
  my $is_definc_init = ( ($internal_flags & 1) ? 1 : 0 );
  if ($is_definc_init) { $behavior = ''; }
  else { &macro_begin_c('IC_C_HEADERS_STDC'); }
  if (!$emitted_c_stdc)
  { $emitted_c_stdc = 1;
    my $dest = SH_PART_SLANG;
    &cop_message_checking('for ANSI-compliant C header files', $dest);
    &cop_c_compile($resultid, $behavior, '#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <float.h>', 1 | 2 | 8);
    &emit_sh($dest, 'ic_c_stdc=$i_result', '', 'case $ic_c_stdc in', '1)');
    &sh_c_header_egrep('string.h', 'memchr', 1);
    &emit_sh($dest, 'case $i_result in', '  0) ic_c_stdc=0 ;;', 'esac', ';;',
      'esac', '', 'case $ic_c_stdc in', '1)');
    &sh_c_header_egrep('stdlib.h', 'free', 1);
    &emit_sh($dest, 'case $i_result in', '  0) ic_c_stdc=0 ;;', 'esac', ';;',
      'esac', '', 'case $ic_c_stdc in', '1)');
    &cop_c_run($resultid, $behavior, 1, '', '#include <ctype.h>
#if ((\' \' & 0x0FF) == 0x020)
# define ISLOWER(c) (\'a\' <= (c) && (c) <= \'z\')
# define TOUPPER(c) (ISLOWER(c) ? \'A\' + ((c) - \'a\') : (c))
#else
# define ISLOWER(c) ((\'a\' <= (c) && (c) <= \'i\') || (\'j\' <= (c) && (c) <= \'r\') || (\'s\' <= (c) && (c) <= \'z\'))
# define TOUPPER(c) (ISLOWER(c) ? ((c) | 0x40) : (c))
#endif

#define XOR(e, f) (((e) && !(f)) || (!(e) && (f)))
int main()
{ int i;
  for (i = 0; i < 256; i++)
    if (XOR(islower(i), ISLOWER(i))
        || toupper(i) != TOUPPER(i))
      return(2);
  return(0);
}', 1 | 2 | 4 | 8);
    &emit_sh($dest, 'case $i_result in', '  0) ic_c_stdc=0 ;;', 'esac', ';;',
      'esac', '', 'case $ic_c_stdc in', '  0) i_x=no ;;',
      '  *) i_x=yes ;;', 'esac');
    &cop_message_checkresult('$i_x', $dest);
  }
  if ($is_definc_init)
  { &emit_sh(SH_PART_SLANG,
      'echo "#define ' . $resultid . ' $ic_c_stdc" >>$ic_int_config_h');
  }
  else
  { my $csflags = 2;
    if ($resultid eq 'STDC_HEADERS') { $csflags |= 1; }
    &cop_substitute($resultid, '$ic_c_stdc', $behavior, $csflags);
    &macro_end();
  }
}

sub sh_egrep
{ # shell test code taken from autoconf-2.53b/lib/autoconf/programs.m4,
  # AC_PROG_EGREP
  if (!$emitted_egrep)
  { $emitted_egrep = 1;
    # &cop_message_checking('how to run egrep', SH_PART_INIT);
    &emit_sh(SH_PART_INIT, "if echo a | (grep -E '(a|b)') >&$fd_sh_null 2>&1",
      "  then ic_egrep='grep -E'", "  else ic_egrep='egrep'", "fi");
    # &cop_message_checkresult('$ic_egrep', SH_PART_INIT);
  }
  return('$ic_egrep');
}

sub sh_c_cpp_egrep
{ my ($code, $text, $dest) = @_;
  &sh_c_write_testfile($code, $dest);
  $need_c_cpp = 1;
  &emit_sh($dest, 'if $ic_CPP $ic_CPPFLAGS0 $ic_testfile.c 2>&' . &sh_dbgfd() .
    ' | ' . &sh_egrep() . ' "' . $text . '" >&' . $fd_sh_null . ' 2>&' .
    &sh_dbgfd() . '; then', '  i_result=1', 'else', '  i_result=0', 'fi');
}

sub sh_c_header_egrep
{ my ($header, $text, $internal_flags) = @_;
  my $is_definc_init = ( ($internal_flags & 1) ? 1 : 0 );
  my $dest = ( ($is_definc_init) ? SH_PART_SLANG : SH_PART_CODE );
  &sh_c_cpp_egrep("#include <$header>", $text, $dest);
  if ($debug & 16)
  { &emit_sh($dest, &sh_debug("C header egrep: $header, $text, \$i_result")); }
}

sub cop_c_library
{ # Test algorithm similar to autoconf-2.52/lib/autoconf/libs.m4, AC_CHECK_LIB
  my ($library, $function, $id, $behavior) = @_;
  my $cclflags;
  if ($id eq '') { $id = $idfoofix{'have-c-library'} . &convname($library); }
  if ($id =~ m/^HAVE\_/) { $cclflags = 4; } # CHECKME: HAVE_ or have-c-library?
  else { $cclflags = 0; }
  &macro_begin_c('IC_C_LIBRARY');
  &cop_message_checking("for function $function in library $library");
  &cop_c_link($id, $behavior, $library, '#include "' . $ic_int_config_h . '"
#ifdef __cplusplus
extern "C"
#endif
char ' . $function . '();
int main()
{ ' . $function . '();
  return(0);
}', $cclflags);
  &emit_sh(SH_PART_CODE, 'case $i_result in', '  0) i_x=no ;;',
    '  *) i_x=yes ;;', 'esac');
  &cop_message_checkresult('$i_x');
  &macro_end();
}

sub split_types
{ return(&split_any($_[0], $_[1], 'types'));
}

sub cop_c_types
{ # Test algorithm similar to autoconf-2.52/lib/autoconf/types.m4,
  # _AC_CHECK_TYPE_NEW
  my ($types, $behavior) = @_;
  if ($behavior eq '') { $behavior = 0; }
  if (!&will_store_result__behavior('IC_C_TYPES', $behavior)) { return; }
  &macro_begin_c('IC_C_TYPES');
  my @t = &split_targets('IC_C_TYPES', $types);
  my %default = ('mode_t' => 'int', 'off_t' => 'long', 'size_t' => 'unsigned',
    'pid_t' => 'int', 'gid_t' => 'int', 'uid_t' => 'int', 'ptrdiff_t' =>'int');
  foreach my $type (@t)
  { my $repl = $default{$type};
    if ( ($repl eq '') and ($behavior == 0) )
    { &show_warning('IC_C_TYPES: no replacement type known for "' . $type .
        '" and behavior is 0; skipping test');
      next;
    }
    my $id = $idfoofix{'have-c-type'} . &convname($type);
    &cop_message_checking('whether the C type ' . $type . ' exists');
    &cop_c_compile($id, $behavior,
'#include "' . $ic_int_config_h . '"
' . &c_definc() . '
int main()
{ if ((' . $type . ' *) 0) return(0);
  if (sizeof(' . $type . ')) return(0);
  return(0);
}', 4);
    &emit_sh(SH_PART_CODE, 'if test "x$i_result" = "x0"; then');
    &cop_message_checkresult('no');
    if ($repl ne '') { &cop_emit_config_h("typedef $repl $type;"); }
    &emit_sh(SH_PART_CODE, 'else');
    &cop_message_checkresult('yes');
    &emit_sh(SH_PART_CODE, 'fi');
  }
  &macro_end();
}

sub cop_c_function_mmap
{ # Test algorithm similar to autoconf-2.52/lib/autoconf/functions.m4,
  # AC_FUNC_MMAP
  my ($id, $behavior) = @_;
  if ($id eq '') { $id = 'HAVE_MMAP'; }
  my $ccrflags = ( ($id =~ m/^HAVE\_/) ? 4 : 0 );
  &macro_begin_c('IC_C_FUNCTION_MMAP');
  &cop_message_checking('whether mmap() works');
  # FIXME: add EINTR tests to the C code!
  # IMPLEMENTME: some test parts are irrelevant if a software package uses
  # mmap() with PROT_READ/MAP_PRIVATE; use a flags parameter for
  # IC_C_FUNCTION_MMAP!
  # FIXME: we need an internal check for the getpagesize() function (similar to
  # IC_C_FUNCTIONS)!
  &c_check_headers('stdlib.h', 'sys/param.h');
  &cop_c_run($id, $behavior, 1, '',
'/* Thanks to Mike Haertel and Jim Avera for this test (from GNU autoconf). */
#include "' . $ic_int_config_h . '"
' . &c_definc() . '
#include <fcntl.h>
#include <sys/mman.h>

#if !STDC_HEADERS && !HAVE_STDLIB_H
char *malloc();
#endif

/* This mess was copied from the GNU getpagesize.h. */
#if !HAVE_GETPAGESIZE
/* Assume that all systems that can run configure have sys/param.h. */
# if !HAVE_SYS_PARAM_H
#  define HAVE_SYS_PARAM_H 1
# endif

# ifdef _SC_PAGESIZE
#  define getpagesize() sysconf(_SC_PAGESIZE)
# else /* no _SC_PAGESIZE */
#  if HAVE_SYS_PARAM_H
#   include <sys/param.h>
#   ifdef EXEC_PAGESIZE
#    define getpagesize() EXEC_PAGESIZE
#   else /* no EXEC_PAGESIZE */
#    ifdef NBPG
#     define getpagesize() NBPG * CLSIZE
#     ifndef CLSIZE
#      define CLSIZE 1
#     endif /* no CLSIZE */
#    else /* no NBPG */
#     ifdef NBPC
#      define getpagesize() NBPC
#     else /* no NBPC */
#      ifdef PAGESIZE
#       define getpagesize() PAGESIZE
#      endif /* PAGESIZE */
#     endif /* no NBPC */
#    endif /* no NBPG */
#   endif /* no EXEC_PAGESIZE */
#  else /* no HAVE_SYS_PARAM_H */
#   define getpagesize() 8192	/* punt totally */
#  endif /* no HAVE_SYS_PARAM_H */
# endif /* no _SC_PAGESIZE */

#endif /* no HAVE_GETPAGESIZE */

int
main()
{ char *data, *data2, *data3;
  int i, pagesize;
  int fd;

  pagesize = getpagesize();

  /* First, make a file with some known garbage in it. */
  data = (char *) malloc(pagesize);
  if (!data)
    return(1);
  for (i = 0; i < pagesize; ++i)
    *(data + i) = rand();
  umask(0);
  fd = creat("$ic_testfile.mmap", 0600);
  if (fd < 0)
    return(2);
  if (write(fd, data, pagesize) != pagesize)
    return(3);
  close(fd);

  /* Next, try to mmap the file at a fixed address which already has
     something else allocated at it. If we can, also make sure that
     we see the same garbage. */
  fd = open("$ic_testfile.mmap", O_RDWR);
  if (fd < 0)
    return(4);
  data2 = (char *) malloc(2 * pagesize);
  if (!data2)
    return(5);
  data2 += (pagesize - ((int) data2 & (pagesize - 1))) & (pagesize - 1);
  if (data2 != mmap(data2, pagesize, PROT_READ | PROT_WRITE,
                     MAP_PRIVATE | MAP_FIXED, fd, 0L))
    return(6);
  for (i = 0; i < pagesize; ++i)
    if (*(data + i) != *(data2 + i))
      return(7);

  /* Finally, make sure that changes to the mapped area do not
     percolate back to the file as seen by read(). (This is a bug on
     some variants of i386 svr4.0.) */
  for (i = 0; i < pagesize; ++i)
    *(data2 + i) = *(data2 + i) + 1;
  data3 = (char *) malloc(pagesize);
  if (!data3)
    return(8);
  if (read(fd, data3, pagesize) != pagesize)
    return(9);
  for (i = 0; i < pagesize; ++i)
    if (*(data + i) != *(data3 + i))
      return(10);
  close(fd);
  return(0);
}', 1 | $ccrflags);
  &emit_sh(SH_PART_CODE, 'case $i_result in', '  0) i_x=no ;;',
    '  *) i_x=yes ;;', 'esac');
  &cop_message_checkresult('$i_x');
  &macro_end();
}


# Section: Sub-Configuration

sub cop_sub_add
{ my ($kinds, $dirs, $extra) = @_;
  $using_sub_add = 1;
  my %known = ('configure' => 1, 'make-binary' => 2, 'make-clean' => 2,
    'dist' => 1);
  my @k = &split_any('IC_SUB_ADD', $kinds, 'kinds');
  my @s = &split_any('IC_SUB_ADD', $dirs, 'sub-directories');
  foreach my $one (@k)
  { my $v = $known{$one};
    if ($v == 0) { &fatal_error("IC_SUB_ADD: unknown kind \"$one\""); }
    if ( ($v == 2) and ($extra eq '') )
    { &fatal_error('IC_SUB_ADD: missing Makefile name as third parameter'); }
    if ($one eq 'dist') { push(@subconf_dist, @s); }
  }
  # IMPLEMENTME!
}

sub cop_sub_begin
{ &deny_toplevelconf('IC_SUB_BEGIN');
  $is_subconf = 1;
  # IMPLEMENTME!
}

sub cop_sub_end
{ &deny_toplevelconf('IC_SUB_END');
  &sh_end();
}
